from __future__ import annotations

import collections
import traceback
from collections import defaultdict
from typing import DefaultDict, Dict, List, Mapping

import pyarrow as pa

from chalk._lsp.error_builder import FeatureClassErrorBuilder, LSPErrorBuilder, ResolverErrorBuilder
from chalk.features import FeatureSetBase
from chalk.features._encoding.serialized_dtype import deserialize_dtype
from chalk.features.resolver import RESOLVER_REGISTRY
from chalk.parsed._metadata import (
    METADATA_STRING_TO_FEATURE_ATTRIBUTE_MAP,
    METADATA_STRING_TO_RESOLVER_ATTRIBUTE_MAP,
    MissingMetadataResolver,
    build_feature_log_for_each_severity,
    build_resolver_log_for_each_severity,
    get_missing_metadata,
    validate_feature_metadata,
)
from chalk.parsed.duplicate_input_gql import (
    GraphLogSeverity,
    ProjectSettingsGQL,
    UpdateGraphError,
    UpsertFeatureGQL,
    UpsertGraphGQL,
    UpsertResolverGQL,
)
from chalk.utils.duration import parse_chalk_duration
from chalk.utils.string import add_quotes, oxford_comma_list


class ClientLogBuilder:
    def __init__(self):
        super().__init__()
        self._logs: List[UpdateGraphError] = []

    def add_log(self, header: str, subheader: str, severity: GraphLogSeverity):
        self._logs.append(UpdateGraphError(header=header, subheader=subheader, severity=severity))

    def add_error(self, header: str, subheader: str):
        self.add_log(header=header, subheader=subheader, severity=GraphLogSeverity.ERROR)

    def add_warning(self, header: str, subheader: str):
        self.add_log(header=header, subheader=subheader, severity=GraphLogSeverity.WARNING)

    def add_info(self, header: str, subheader: str):
        self.add_log(header=header, subheader=subheader, severity=GraphLogSeverity.INFO)

    def get_logs(self) -> List[UpdateGraphError]:
        return self._logs


def _validate_primary_key(
    feature: UpsertFeatureGQL,
    builder: ClientLogBuilder,
    lsp_builder: FeatureClassErrorBuilder,
):
    if feature.scalarKind is not None and feature.scalarKind.primary:
        assert feature.scalarKind.dtype is not None, f"The serialized dtype is not set for feature '{feature.id.fqn}'"
        pa_dtype = deserialize_dtype(feature.scalarKind.dtype)
        if (
            not pa.types.is_integer(pa_dtype)
            and not pa.types.is_large_string(pa_dtype)
            and not pa.types.is_string(pa_dtype)
        ):
            lsp_builder.add_diagnostic(
                label="invalid type",
                message=f"Primary keys must be integers or strings. Feature '{feature.id.fqn}' has a type of '{pa_dtype}'",
                range=lsp_builder.annotation_range(feature.id.attributeName or feature.id.name),
                code="23",
            )
            builder.add_error(
                header=f'Invalid primary key type "{feature.id.fqn}"',
                subheader=(
                    f"Primary keys must be integers or strings. " f'Feature "{feature.id.fqn}" has a type of {pa_dtype}'
                ),
            )


def _validate_max_staleness(
    feature: UpsertFeatureGQL,
    builder: ClientLogBuilder,
    lsp_builder: FeatureClassErrorBuilder,
):
    if feature.maxStaleness is not None and feature.maxStaleness != "infinity":
        try:
            parse_chalk_duration(feature.maxStaleness)
        except ValueError as e:
            builder.add_error(
                header=f'Could not parse max_staleness for feature "{feature.id.fqn}"',
                subheader=f"Invalid max staleness. {e.args[0]}",
            )


def _validate_etl_to_online(
    feature: UpsertFeatureGQL,
    builder: ClientLogBuilder,
    lsp_builder: FeatureClassErrorBuilder,
):
    if feature.etlOfflineToOnline and feature.maxStaleness is None:
        builder.add_error(
            header=f'Missing max staleness for "{feature.id.fqn}"',
            subheader=f'The feature "{feature.id.fqn}" is set to ETL to online, but doesn\'t specify a max staleness. Any ETL to online would be immediately invalidated.',
        )


def _validate_no_feature_times_as_input(
    fqn_to_feature: Mapping[str, UpsertFeatureGQL],
    resolver: UpsertResolverGQL,
    builder: ClientLogBuilder,
    lsp_builder: ResolverErrorBuilder,
):
    for i, inp in enumerate(resolver.inputs or []):
        if inp.underlying.fqn not in fqn_to_feature:
            message = (
                f"Resolver '{resolver.fqn}' requires an unrecognized feature '{inp.underlying.fqn}'. "
                + f"This may have happened since the import of feature class '{inp.underlying.namespace}' may have failed"
            )
            lsp_builder.add_diagnostic(
                message=message,
                code="135",
                range=lsp_builder.function_arg_annotation_by_index(i),
                label="invalid resolver input",
            )
            builder.add_error(
                header=message,
                subheader="Please check the feature inputs to this resolver were imported correctly.",
            )
        elif fqn_to_feature[inp.underlying.fqn].featureTimeKind is not None and resolver.kind != "offline":
            lsp_builder.add_diagnostic(
                message=(
                    f'The resolver "{resolver.fqn}" takes as input the\n'
                    f'feature "{inp.underlying.fqn}", a feature_time() type.\n'
                    f"Feature times can be returned from {resolver.kind} resolvers to\n"
                    "indicate data from the past, but cannot be accepted\n"
                    "as inputs."
                ),
                code="136",
                range=lsp_builder.function_arg_annotation_by_index(i),
                label="invalid resolver input",
            )
            builder.add_error(
                header=f"Feature times cannot be accepted as input to {resolver.kind} resolvers.",
                subheader=(
                    f'The resolver "{resolver.fqn}" takes as input the\n'
                    f'feature "{inp.underlying.fqn}", a feature_time() type.\n'
                    f"Feature times can be returned from {resolver.kind} resolvers to\n"
                    "indicate data from the past, but cannot be accepted\n"
                    "as inputs."
                ),
            )


def _validate_feature_names_unique(features: List[UpsertFeatureGQL], builder: ClientLogBuilder):
    counter: DefaultDict[str, int] = defaultdict(lambda: 0)
    for r in features:
        counter[r.id.fqn] += 1

    for fqn, count in counter.items():
        if count > 1:
            builder.add_error(
                header="Duplicate feature names",
                subheader=(
                    f'There are {count} features with the same name of "{fqn}". All features require a '
                    f"distinct name"
                ),
            )


def _validate_resolver_names_unique(resolvers: List[UpsertResolverGQL], builder: ClientLogBuilder):
    counter: DefaultDict[str, int] = defaultdict(lambda: 0)
    for r in resolvers:
        counter[r.fqn.split(".")[-1]] += 1

    for name, count in counter.items():
        if count > 1:
            builder.add_error(
                header="Duplicate resolver names",
                subheader=(
                    f'There are {count} resolvers with the same name of "{name}". All resolvers require a '
                    f"distinct name"
                ),
            )


def _validate_resolver_input(
    singleton_namespaces: set[str],
    resolver: UpsertResolverGQL,
    builder: ClientLogBuilder,
    lsp_builder: ResolverErrorBuilder,
):
    namespaces = []
    ns_to_arg_num: Dict[str, int] = {}

    for idx, input_ in enumerate(resolver.inputs or []):
        f = input_.path[0].parent if input_.path is not None and len(input_.path) > 0 else input_.underlying
        if f.namespace not in singleton_namespaces:
            ns_to_arg_num[f.namespace] = idx
            if f.namespace not in namespaces:
                namespaces.append(f.namespace)

    if len(namespaces) > 1:
        arg_num = list(ns_to_arg_num.values())[0]
        d = lsp_builder.add_diagnostic(
            message=f"""All inputs to a resolver should be rooted in the same namespace.
The resolver "{resolver.fqn}" takes inputs in the namespaces {oxford_comma_list(add_quotes(namespaces))}.

If you require features from many feature classes, reference them via their relationships, such as:
  User.bank_account.title,
  User.full_name""",
            code="131",
            range=lsp_builder.function_arg_annotation_by_index(arg_num),
            label="secondary namespace",
        )
        for ns, arg in ns_to_arg_num.items():
            if arg != arg_num:
                d.with_range(
                    lsp_builder.function_arg_annotation_by_index(arg),
                    label=f"other namespace {ns}",
                )

        builder.add_error(
            header=f'Resolver "{resolver.fqn}" requires features from multiple namespaces.',
            subheader=f"""All inputs to a resolver should be rooted in the same namespace.
The resolver "{resolver.fqn}" takes inputs in the namespaces {oxford_comma_list(add_quotes(namespaces))}.

If you require features from many feature classes, reference them via their relationships, such as:
  User.bank_account.title,
  User.full_name

{resolver.functionDefinition}
""",
        )


def _validate_resolver_output(
    resolver: UpsertResolverGQL, builder: ClientLogBuilder, lsp_builder: ResolverErrorBuilder
):
    output = resolver.output
    features = output.features or []
    dataframes = output.dataframes or []
    if len(features) == 0 and len(dataframes) == 0:
        lsp_builder.add_diagnostic(
            message=f'Resolver "{resolver.fqn}" does not define outputs. All resolvers must have an output',
            code="132",
            range=lsp_builder.function_return_annotation(),
            label="invalid resolver output",
            code_href="https://docs.chalk.ai/docs/python-resolvers#outputs",
        )
        builder.add_error(
            header=f'Resolver "{resolver.fqn}" does not define outputs.',
            subheader="See https://docs.chalk.ai/docs/python-resolvers#outputs for information about valid resolver outputs.",
        )

    if len(dataframes) > 1:
        lsp_builder.add_diagnostic(
            message=f'Resolver "{resolver.fqn}" defines multiple DataFrames as output. At most one is permitted',
            code="133",
            range=lsp_builder.function_return_annotation(),
            label="invalid resolver output",
            code_href="https://docs.chalk.ai/docs/python-resolvers#outputs",
        )
        builder.add_error(
            header=f'Resolver "{resolver.fqn}" defines multiple DataFrames as output.',
            subheader="See https://docs.chalk.ai/docs/python-resolvers#outputs for information about valid resolver outputs.",
        )

    if len(features) > 0 and len(dataframes) > 0:
        lsp_builder.add_diagnostic(
            message=(
                f'Resolver "{resolver.fqn}" returns both relationships and scalar features. '
                f"Only one or the other is permitted."
            ),
            code="134",
            range=lsp_builder.function_return_annotation(),
            label="invalid resolver output",
            code_href="https://docs.chalk.ai/docs/python-resolvers#outputs",
        )
        builder.add_error(
            header=f'Resolver "{resolver.fqn}" returns both relationships and scalar features.',
            subheader="See https://docs.chalk.ai/docs/python-resolvers#outputs for information about valid resolver outputs.",
        )

    # Want to validate this but only have feature id instead of feature
    # if len(resolver.inputs) == 0 and len(dataframes) == 1 and len([pkey for pkey in dataframes[0].columns if pkey.primary]) != 1:
    #     builder.add_error(
    #         header=f'Resolver "{resolver.fqn}" must return a primary feature',
    #         subheader="See https://docs.chalk.ai/docs/python-resolvers#outputs for information about valid resolver outputs.",
    #     )
    # for output_feature in features:
    #     if is_underlying_has_many(output_feature):
    #         if len([pkey for pkey in output_feature.underlying.df.columns if pkey.primary]) != 1:
    #         builder.add_error(
    #             header=f'Resolver "{resolver.fqn}" must return a primary feature in has-many outputs',
    #             subheader="See https://docs.chalk.ai/docs/python-resolvers#outputs for information about valid resolver outputs.",
    #         )
    # Bring this back.


#     if len(features) > 0:
#         namespaces = list(set(f.namespace for f in features))
#         if len(namespaces) > 1:
#             builder.add_error(
#                 header=f'Resolver "{resolver.fqn}" outputs features from multiple namespaces.',
#                 subheader=f"""All outputs of a resolver should be rooted in the same namespace.
# The resolver "{resolver.fqn}" outputs features in the namespaces {oxford_comma_list(namespaces)}.
# See https://docs.chalk.ai/docs/python-resolvers#outputs for information about valid resolver outputs.
#
# {resolver.functionDefinition}
# """,
#             )


# FIXME CHA-66 we should validate that joins are all pkey on pkey joins. no non-primary keys, no self-joins, no constants
def _validate_joins(feature: UpsertFeatureGQL, builder: ClientLogBuilder, lsp_builder: FeatureClassErrorBuilder):
    pass


def _validate_feature_names(
    feature: UpsertFeatureGQL, builder: ClientLogBuilder, lsp_builder: FeatureClassErrorBuilder
):
    if feature.id.name.startswith("__") or feature.id.name.startswith("_chalk"):
        lsp_builder.add_diagnostic(
            message="Feature names cannot begin with '_chalk' or '__'.",
            range=lsp_builder.property_range(feature.id.attributeName or feature.id.name),
            label="protected name",
            code="24",
        )
        builder.add_error(
            header="Feature uses protected name",
            subheader="Feature names cannot begin with '_chalk' or '__'.",
        )

    if feature.id.namespace.startswith("__") or feature.id.namespace.startswith("_chalk"):
        lsp_builder.add_diagnostic(
            message="Feature classes cannot have names that begin with '_chalk' or '__'.",
            label="protected namespace",
            range=lsp_builder.decorator_kwarg_value_range("name") or lsp_builder.class_definition_range(),
            code="25",
        )
        builder.add_error(
            header="Feature class uses protected namespace",
            subheader=(
                f'The feature "{feature.id.fqn}" belongs to the protected namespace "{feature.id.namespace}". '
                f'Feature namespaces cannot begin with "_chalk" or "__". Please rename this feature set.'
            ),
        )


def _validate_metadata(
    features: list[UpsertFeatureGQL],
    resolvers: list[UpsertResolverGQL],
    builder: ClientLogBuilder,
    request_config: ProjectSettingsGQL | None,
):
    if not (request_config and request_config.validation):
        return

    if request_config.validation.feature and request_config.validation.feature.metadata:
        namespace_to_features = collections.defaultdict(list)
        for f in features:
            namespace_to_features[f.id.namespace].append(f)

        for ns_features in namespace_to_features.values():
            wf = validate_feature_metadata(
                settings=request_config.validation.feature.metadata, namespace_features=ns_features
            )
            build_feature_log_for_each_severity(builder=builder, missing_metadata_features=wf)
    if request_config.validation.resolver and request_config.validation.resolver.metadata:
        missing_resolver_metadatas: list[MissingMetadataResolver] = []
        for r in resolvers:
            metadata = get_missing_metadata(
                entity=r,
                attribute_map=METADATA_STRING_TO_RESOLVER_ATTRIBUTE_MAP,
                settings=request_config.validation.resolver.metadata,
            )
            missing_resolver_metadatas.append(MissingMetadataResolver(resolver=r, missing_metadata=metadata))
        build_resolver_log_for_each_severity(builder, missing_resolver_metadatas)


def _validate_metadata_config(
    builder: ClientLogBuilder,
    request_config: ProjectSettingsGQL | None,
):
    if not (request_config and request_config.validation):
        return

    severities_lower = [e.lower() for e in GraphLogSeverity]

    if request_config.validation.feature and request_config.validation.feature.metadata:
        for missing_metadata in request_config.validation.feature.metadata:
            metadata_name = missing_metadata.name
            severity = missing_metadata.missing

            severity_upper = severity.upper()
            try:
                GraphLogSeverity(severity_upper)
            except ValueError:
                severity_choices = '" or "'.join(severities_lower)
                builder.add_warning(
                    header=f'Found invalid log severity "{severity}" config for missing metadata',
                    subheader=(
                        f'The required feature metadata "{metadata_name}" is associated with an invalid log severity "{severity}".'
                        f' Please use "{severity_choices}" in chalk.yml'
                    ),
                )

            if metadata_name not in METADATA_STRING_TO_FEATURE_ATTRIBUTE_MAP:
                builder.add_warning(
                    header=f'Found invalid feature metadata "{metadata_name}" in config',
                    subheader=(
                        f'The required metadata "{metadata_name}" is not a valid feature metadata.'
                        f" Please consider removing it from chalk.yml"
                    ),
                )

    if request_config.validation.resolver and request_config.validation.resolver.metadata:
        for missing_metadata in request_config.validation.resolver.metadata:
            metadata_name = missing_metadata.name
            severity = missing_metadata.missing

            severity_upper = severity.upper()
            try:
                GraphLogSeverity(severity_upper)
            except ValueError:
                severity_choices = '" or "'.join(severities_lower)
                builder.add_warning(
                    header=f'Found invalid log severity "{severity}" config for missing metadata',
                    subheader=(
                        f'The required feature metadata "{metadata_name}" is associated with an invalid log severity "{severity}".'
                        f' Please use "{severity_choices}" in chalk.yml'
                    ),
                )

            if metadata_name not in METADATA_STRING_TO_RESOLVER_ATTRIBUTE_MAP:
                builder.add_warning(
                    header=f'Found invalid feature metadata "{metadata_name}" in config',
                    subheader=(
                        f'The required metadata "{metadata_name}" is not a valid feature metadata.'
                        f" Please consider removing it from chalk.yml"
                    ),
                )


def _validate_namespace_primary_key(
    namespace: str,
    features: List[UpsertFeatureGQL],
    builder: ClientLogBuilder,
    singleton_namespaces: set[str],
):
    if namespace in singleton_namespaces:
        return

    primary_features = list(f for f in features if f.scalarKind and f.scalarKind.primary)

    if len(primary_features) == 0:
        builder.add_error(
            header=f"Feature set '{namespace}' is missing a primary feature",
            subheader=f"Please add an 'int' or 'str' feature to '{namespace}', annotated with '= feature(primary=True)'",
        )
    elif len(primary_features) > 1:
        names = ", ".join([f.id.name for f in primary_features])
        builder.add_error(
            header=f"Feature set '{namespace}' has too many primary features",
            subheader=f"Found primary features: {names}. Composite primary keys are not supported. Please mark only a single feature as primary.",
        )


def validate_graph(request: UpsertGraphGQL) -> List[UpdateGraphError]:
    """Beware: errors here will only be acted upon `chalk apply --branch`.
    Regular deployments will have their graph errors acted upon by the server.
    """
    singleton_namespaces = {c.name for c in request.featureClasses or [] if c.isSingleton}

    namespaces: Dict[str, List[UpsertFeatureGQL]] = defaultdict(list)

    for feature in request.features or []:
        namespaces[feature.id.namespace].append(feature)

    builder = ClientLogBuilder()

    # Validate the features
    _validate_feature_names_unique(request.features or [], builder)
    _validate_metadata_config(builder, request.config)
    _validate_metadata(request.features or [], request.resolvers or [], builder, request.config)

    for namespace, features in namespaces.items():
        _validate_namespace_primary_key(
            namespace=namespace,
            features=features,
            builder=builder,
            singleton_namespaces=singleton_namespaces,
        )

    garbage_feature_builder = FeatureClassErrorBuilder(uri="garbage.py", namespace="garbage", node=None)
    for feature in request.features or []:
        lsp_builder = (
            FeatureSetBase.registry[feature.id.namespace].__chalk_error_builder__
            if feature.id.namespace in FeatureSetBase.registry and LSPErrorBuilder.lsp
            else garbage_feature_builder
        )
        _validate_primary_key(feature, builder, lsp_builder)
        _validate_max_staleness(feature, builder, lsp_builder)
        _validate_joins(feature, builder, lsp_builder)
        _validate_etl_to_online(feature, builder, lsp_builder)
        _validate_feature_names(feature, builder, lsp_builder)

    # Validate the resolvers
    fqn_to_feature = {f.id.fqn: f for f in request.features or []}

    _validate_resolver_names_unique(request.resolvers or [], builder)
    garbage_resolver_builder = ResolverErrorBuilder(fn=None)
    for resolver in request.resolvers or []:
        try:
            lsp_builder = garbage_resolver_builder
            if LSPErrorBuilder.lsp:
                maybe_resolver = RESOLVER_REGISTRY.get_resolver(resolver.fqn)
                if maybe_resolver is not None:
                    if maybe_resolver.is_sql_file_resolver:
                        continue
                    lsp_builder = maybe_resolver.lsp_builder
            _validate_resolver_input(
                singleton_namespaces=singleton_namespaces,
                resolver=resolver,
                builder=builder,
                lsp_builder=lsp_builder,
            )
            _validate_resolver_output(resolver, builder, lsp_builder)

            # TODO we still allow this
            # _validate_resolver_feature_cycles(fqn_to_feature=fqn_to_feature, resolver=resolver, builder=builder)

            # TODO Some customers currently still do stuff like:
            # >>> def some_resolver(uid: User.id) -> DataFrame[Transaction]: ....
            # So don't even warn about it
            # _validate_resolver_input_and_output_namespace(resolver, builder)

            _validate_no_feature_times_as_input(
                fqn_to_feature=fqn_to_feature, resolver=resolver, builder=builder, lsp_builder=lsp_builder
            )
        except Exception:
            err = traceback.format_exc()
            builder.add_error(
                header=f'Failed to validate resolver "{resolver.fqn}"',
                subheader=f"Please check the resolver for syntax errors.\nRaw error: {err}",
            )

    return builder.get_logs()



from chalk._gen.chalk.auth.v1 import permissions_pb2 as _permissions_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class GetOverviewSummaryMetricsRequest(_message.Message):
    __slots__ = ("range_start", "range_end")
    RANGE_START_FIELD_NUMBER: _ClassVar[int]
    RANGE_END_FIELD_NUMBER: _ClassVar[int]
    range_start: str
    range_end: str
    def __init__(self, range_start: _Optional[str] = ..., range_end: _Optional[str] = ...) -> None: ...

class OverviewSummaryMetric(_message.Message):
    __slots__ = ("name", "value")
    NAME_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    name: str
    value: float
    def __init__(self, name: _Optional[str] = ..., value: _Optional[float] = ...) -> None: ...

class GetOverviewSummaryMetricsResponse(_message.Message):
    __slots__ = ("metrics",)
    METRICS_FIELD_NUMBER: _ClassVar[int]
    metrics: _containers.RepeatedCompositeFieldContainer[OverviewSummaryMetric]
    def __init__(self, metrics: _Optional[_Iterable[_Union[OverviewSummaryMetric, _Mapping]]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/deploy.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.artifacts.v1 import export_pb2 as chalk_dot_artifacts_dot_v1_dot_export__pb2
from chalk._gen.chalk.auth.v1 import audit_pb2 as chalk_dot_auth_dot_v1_dot_audit__pb2
from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.common.v1 import chalk_error_pb2 as chalk_dot_common_dot_v1_dot_chalk__error__pb2
from chalk._gen.chalk.graph.v1 import graph_pb2 as chalk_dot_graph_dot_v1_dot_graph__pb2
from chalk._gen.chalk.server.v1 import deployment_pb2 as chalk_dot_server_dot_v1_dot_deployment__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1c\x63halk/server/v1/deploy.proto\x12\x0f\x63halk.server.v1\x1a\x1f\x63halk/artifacts/v1/export.proto\x1a\x19\x63halk/auth/v1/audit.proto\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a!chalk/common/v1/chalk_error.proto\x1a\x1a\x63halk/graph/v1/graph.proto\x1a chalk/server/v1/deployment.proto\x1a\x1bgoogle/protobuf/empty.proto"\x97\x01\n\x13\x44\x65ployBranchRequest\x12\x1f\n\x0b\x62ranch_name\x18\x01 \x01(\tR\nbranchName\x12!\n\x0creset_branch\x18\x02 \x01(\x08R\x0bresetBranch\x12\x18\n\x07\x61rchive\x18\x03 \x01(\x0cR\x07\x61rchive\x12"\n\ris_hot_deploy\x18\x04 \x01(\x08R\x0bisHotDeploy"\x89\x02\n\x14\x44\x65ployBranchResponse\x12#\n\rdeployment_id\x18\x01 \x01(\tR\x0c\x64\x65ploymentId\x12\x34\n\x05graph\x18\x02 \x01(\x0b\x32\x15.chalk.graph.v1.GraphB\x02\x18\x01H\x00R\x05graph\x88\x01\x01\x12H\n\x11\x64\x65ployment_errors\x18\x03 \x03(\x0b\x32\x1b.chalk.common.v1.ChalkErrorR\x10\x64\x65ploymentErrors\x12\x37\n\x06\x65xport\x18\x04 \x01(\x0b\x32\x1a.chalk.artifacts.v1.ExportH\x01R\x06\x65xport\x88\x01\x01\x42\x08\n\x06_graphB\t\n\x07_export"\x92\x02\n\'CreateBranchFromSourceDeploymentRequest\x12\x1f\n\x0b\x62ranch_name\x18\x01 \x01(\tR\nbranchName\x12.\n\x12source_branch_name\x18\x02 \x01(\tH\x00R\x10sourceBranchName\x12\x32\n\x14source_deployment_id\x18\x03 \x01(\tH\x00R\x12sourceDeploymentId\x12X\n\x1b\x63urrent_mainline_deployment\x18\x04 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x00R\x19\x63urrentMainlineDeploymentB\x08\n\x06source"\x91\x02\n(CreateBranchFromSourceDeploymentResponse\x12#\n\rdeployment_id\x18\x01 \x01(\tR\x0c\x64\x65ploymentId\x12H\n\x11\x64\x65ployment_errors\x18\x02 \x03(\x0b\x32\x1b.chalk.common.v1.ChalkErrorR\x10\x64\x65ploymentErrors\x12\x37\n\x06\x65xport\x18\x03 \x01(\x0b\x32\x1a.chalk.artifacts.v1.ExportH\x00R\x06\x65xport\x88\x01\x01\x12\x32\n\x15\x62ranch_already_exists\x18\x04 \x01(\x08R\x13\x62ranchAlreadyExistsB\t\n\x07_export";\n\x14GetDeploymentRequest\x12#\n\rdeployment_id\x18\x01 \x01(\tR\x0c\x64\x65ploymentId"T\n\x15GetDeploymentResponse\x12;\n\ndeployment\x18\x01 \x01(\x0b\x32\x1b.chalk.server.v1.DeploymentR\ndeployment"\xda\x01\n\x16ListDeploymentsRequest\x12\x1b\n\x06\x63ursor\x18\x01 \x01(\tH\x00R\x06\x63ursor\x88\x01\x01\x12\x19\n\x05limit\x18\x02 \x01(\x05H\x01R\x05limit\x88\x01\x01\x12*\n\x0einclude_branch\x18\x03 \x01(\x08H\x02R\rincludeBranch\x88\x01\x01\x12$\n\x0b\x62ranch_name\x18\x04 \x01(\tH\x03R\nbranchName\x88\x01\x01\x42\t\n\x07_cursorB\x08\n\x06_limitB\x11\n\x0f_include_branchB\x0e\n\x0c_branch_name"\x80\x01\n\x17ListDeploymentsResponse\x12=\n\x0b\x64\x65ployments\x18\x01 \x03(\x0b\x32\x1b.chalk.server.v1.DeploymentR\x0b\x64\x65ployments\x12\x1b\n\x06\x63ursor\x18\x02 \x01(\tH\x00R\x06\x63ursor\x88\x01\x01\x42\t\n\x07_cursor"?\n\x18SuspendDeploymentRequest\x12#\n\rdeployment_id\x18\x01 \x01(\tR\x0c\x64\x65ploymentId"X\n\x19SuspendDeploymentResponse\x12;\n\ndeployment\x18\x01 \x01(\x0b\x32\x1b.chalk.server.v1.DeploymentR\ndeployment"v\n\x16ScaleDeploymentRequest\x12#\n\rdeployment_id\x18\x01 \x01(\tR\x0c\x64\x65ploymentId\x12\x37\n\x06sizing\x18\x02 \x01(\x0b\x32\x1f.chalk.server.v1.InstanceSizingR\x06sizing"V\n\x17ScaleDeploymentResponse\x12;\n\ndeployment\x18\x01 \x01(\x0b\x32\x1b.chalk.server.v1.DeploymentR\ndeployment"\x89\x01\n\x14TagDeploymentRequest\x12#\n\rdeployment_id\x18\x01 \x01(\tR\x0c\x64\x65ploymentId\x12\x10\n\x03tag\x18\x02 \x01(\tR\x03tag\x12(\n\rmirror_weight\x18\x03 \x01(\x05H\x00R\x0cmirrorWeight\x88\x01\x01\x42\x10\n\x0e_mirror_weight"\xaa\x01\n\x15TagDeploymentResponse\x12;\n\ndeployment\x18\x01 \x01(\x0b\x32\x1b.chalk.server.v1.DeploymentR\ndeployment\x12\x39\n\x16untagged_deployment_id\x18\x02 \x01(\tH\x00R\x14untaggedDeploymentId\x88\x01\x01\x42\x19\n\x17_untagged_deployment_id"\x1d\n\x1bGetActiveDeploymentsRequest"]\n\x1cGetActiveDeploymentsResponse\x12=\n\x0b\x64\x65ployments\x18\x01 \x03(\x0b\x32\x1b.chalk.server.v1.DeploymentR\x0b\x64\x65ployments2\xad\x07\n\rDeployService\x12`\n\x0c\x44\x65ployBranch\x12$.chalk.server.v1.DeployBranchRequest\x1a%.chalk.server.v1.DeployBranchResponse"\x03\x80}\r\x12\x9c\x01\n CreateBranchFromSourceDeployment\x12\x38.chalk.server.v1.CreateBranchFromSourceDeploymentRequest\x1a\x39.chalk.server.v1.CreateBranchFromSourceDeploymentResponse"\x03\x80}\r\x12\x63\n\rGetDeployment\x12%.chalk.server.v1.GetDeploymentRequest\x1a&.chalk.server.v1.GetDeploymentResponse"\x03\x80}\x0b\x12i\n\x0fListDeployments\x12\'.chalk.server.v1.ListDeploymentsRequest\x1a(.chalk.server.v1.ListDeploymentsResponse"\x03\x80}\x0b\x12x\n\x14GetActiveDeployments\x12,.chalk.server.v1.GetActiveDeploymentsRequest\x1a-.chalk.server.v1.GetActiveDeploymentsResponse"\x03\x80}\x02\x12u\n\x11SuspendDeployment\x12).chalk.server.v1.SuspendDeploymentRequest\x1a*.chalk.server.v1.SuspendDeploymentResponse"\t\x80}\x0c\x8a\xd3\x0e\x02\x08\x02\x12o\n\x0fScaleDeployment\x12\'.chalk.server.v1.ScaleDeploymentRequest\x1a(.chalk.server.v1.ScaleDeploymentResponse"\t\x80}\x0c\x8a\xd3\x0e\x02\x08\x02\x12i\n\rTagDeployment\x12%.chalk.server.v1.TagDeploymentRequest\x1a&.chalk.server.v1.TagDeploymentResponse"\t\x80}\x0c\x8a\xd3\x0e\x02\x08\x02\x42\x94\x01\n\x13\x63om.chalk.server.v1B\x0b\x44\x65ployProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.deploy_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\013DeployProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_DEPLOYBRANCHRESPONSE"].fields_by_name["graph"]._options = None
    _globals["_DEPLOYBRANCHRESPONSE"].fields_by_name["graph"]._serialized_options = b"\030\001"
    _globals["_DEPLOYSERVICE"].methods_by_name["DeployBranch"]._options = None
    _globals["_DEPLOYSERVICE"].methods_by_name["DeployBranch"]._serialized_options = b"\200}\r"
    _globals["_DEPLOYSERVICE"].methods_by_name["CreateBranchFromSourceDeployment"]._options = None
    _globals["_DEPLOYSERVICE"].methods_by_name["CreateBranchFromSourceDeployment"]._serialized_options = b"\200}\r"
    _globals["_DEPLOYSERVICE"].methods_by_name["GetDeployment"]._options = None
    _globals["_DEPLOYSERVICE"].methods_by_name["GetDeployment"]._serialized_options = b"\200}\013"
    _globals["_DEPLOYSERVICE"].methods_by_name["ListDeployments"]._options = None
    _globals["_DEPLOYSERVICE"].methods_by_name["ListDeployments"]._serialized_options = b"\200}\013"
    _globals["_DEPLOYSERVICE"].methods_by_name["GetActiveDeployments"]._options = None
    _globals["_DEPLOYSERVICE"].methods_by_name["GetActiveDeployments"]._serialized_options = b"\200}\002"
    _globals["_DEPLOYSERVICE"].methods_by_name["SuspendDeployment"]._options = None
    _globals["_DEPLOYSERVICE"].methods_by_name[
        "SuspendDeployment"
    ]._serialized_options = b"\200}\014\212\323\016\002\010\002"
    _globals["_DEPLOYSERVICE"].methods_by_name["ScaleDeployment"]._options = None
    _globals["_DEPLOYSERVICE"].methods_by_name[
        "ScaleDeployment"
    ]._serialized_options = b"\200}\014\212\323\016\002\010\002"
    _globals["_DEPLOYSERVICE"].methods_by_name["TagDeployment"]._options = None
    _globals["_DEPLOYSERVICE"].methods_by_name[
        "TagDeployment"
    ]._serialized_options = b"\200}\014\212\323\016\002\010\002"
    _globals["_DEPLOYBRANCHREQUEST"]._serialized_start = 269
    _globals["_DEPLOYBRANCHREQUEST"]._serialized_end = 420
    _globals["_DEPLOYBRANCHRESPONSE"]._serialized_start = 423
    _globals["_DEPLOYBRANCHRESPONSE"]._serialized_end = 688
    _globals["_CREATEBRANCHFROMSOURCEDEPLOYMENTREQUEST"]._serialized_start = 691
    _globals["_CREATEBRANCHFROMSOURCEDEPLOYMENTREQUEST"]._serialized_end = 965
    _globals["_CREATEBRANCHFROMSOURCEDEPLOYMENTRESPONSE"]._serialized_start = 968
    _globals["_CREATEBRANCHFROMSOURCEDEPLOYMENTRESPONSE"]._serialized_end = 1241
    _globals["_GETDEPLOYMENTREQUEST"]._serialized_start = 1243
    _globals["_GETDEPLOYMENTREQUEST"]._serialized_end = 1302
    _globals["_GETDEPLOYMENTRESPONSE"]._serialized_start = 1304
    _globals["_GETDEPLOYMENTRESPONSE"]._serialized_end = 1388
    _globals["_LISTDEPLOYMENTSREQUEST"]._serialized_start = 1391
    _globals["_LISTDEPLOYMENTSREQUEST"]._serialized_end = 1609
    _globals["_LISTDEPLOYMENTSRESPONSE"]._serialized_start = 1612
    _globals["_LISTDEPLOYMENTSRESPONSE"]._serialized_end = 1740
    _globals["_SUSPENDDEPLOYMENTREQUEST"]._serialized_start = 1742
    _globals["_SUSPENDDEPLOYMENTREQUEST"]._serialized_end = 1805
    _globals["_SUSPENDDEPLOYMENTRESPONSE"]._serialized_start = 1807
    _globals["_SUSPENDDEPLOYMENTRESPONSE"]._serialized_end = 1895
    _globals["_SCALEDEPLOYMENTREQUEST"]._serialized_start = 1897
    _globals["_SCALEDEPLOYMENTREQUEST"]._serialized_end = 2015
    _globals["_SCALEDEPLOYMENTRESPONSE"]._serialized_start = 2017
    _globals["_SCALEDEPLOYMENTRESPONSE"]._serialized_end = 2103
    _globals["_TAGDEPLOYMENTREQUEST"]._serialized_start = 2106
    _globals["_TAGDEPLOYMENTREQUEST"]._serialized_end = 2243
    _globals["_TAGDEPLOYMENTRESPONSE"]._serialized_start = 2246
    _globals["_TAGDEPLOYMENTRESPONSE"]._serialized_end = 2416
    _globals["_GETACTIVEDEPLOYMENTSREQUEST"]._serialized_start = 2418
    _globals["_GETACTIVEDEPLOYMENTSREQUEST"]._serialized_end = 2447
    _globals["_GETACTIVEDEPLOYMENTSRESPONSE"]._serialized_start = 2449
    _globals["_GETACTIVEDEPLOYMENTSRESPONSE"]._serialized_end = 2542
    _globals["_DEPLOYSERVICE"]._serialized_start = 2545
    _globals["_DEPLOYSERVICE"]._serialized_end = 3486
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING, Mapping

from chalk.sql._internal.incremental import IncrementalSettings

if TYPE_CHECKING:
    from chalk.sql.finalized_query import Finalizer


@dataclass
class SQLResolverSettings:
    finalizer: Finalizer
    incremental_settings: IncrementalSettings | None
    fields_root_fqn: Mapping[str, str]  # column name -> root fqn of output feature
    params_to_root_fqn: Mapping[str, str]  # escaped param name -> root fqn of input feature



from __future__ import annotations

import contextlib
import os
import threading
import time
import types
from typing import TYPE_CHECKING, Any, Mapping, Union, cast

from chalk.utils._ddtrace_version import can_use_datadog_statsd, can_use_ddtrace
from chalk.utils.environment_parsing import env_var_bool

if TYPE_CHECKING:
    import ddtrace.context

if can_use_ddtrace and can_use_datadog_statsd:
    import ddtrace
    from datadog.dogstatsd.base import statsd

    def safe_set_gauge(gauge: str, value: int | float):
        statsd.gauge(gauge, value)

    def safe_incr(counter: str, value: int | float, tags: list[str] | None = None):
        statsd.increment(counter, value, tags)

    def safe_distribution(counter: str, value: int | float, tags: list[str] | None = None):
        statsd.distribution(counter, value, tags)

    @contextlib.contextmanager
    def safe_trace(span_id: str, attributes: Mapping[str, str] | None = None):
        if not ddtrace.tracer.enabled:
            yield
            return
        if (current_ctx := ddtrace.tracer.current_trace_context()) is None:
            yield
            return
        if (priority := current_ctx.sampling_priority) is not None and priority <= 0:
            # If a priority is negative, then it won't be sampled
            # See https://github.com/DataDog/dd-trace-py/blob/09edef713bf9f0ab30f554bf7765d7a7c2ed6f30/ddtrace/constants.py#L74
            # Not sure what a priority=None means
            yield
            return
        if attributes is None:
            attributes = {}
        attributes = dict(attributes)
        attributes["thread_id"] = str(threading.get_native_id())
        with ddtrace.tracer.trace(name=span_id) as span:
            if hasattr(span, "_ignore_exception"):
                span._ignore_exception(GeneratorExit)  # pyright: ignore [reportPrivateUsage, reportArgumentType]
                from chalk.sql._internal.sql_source import UnsupportedEfficientExecutionError

                span._ignore_exception(  # pyright: ignore [reportPrivateUsage]
                    UnsupportedEfficientExecutionError  # pyright: ignore [reportArgumentType]
                )
            if attributes:
                span.set_tags(cast(Any, attributes))
            yield

    def safe_add_metrics(metrics: Mapping[str, Union[int, float]]):
        span = ddtrace.tracer.current_span()
        if span:
            span.set_metrics(cast(Any, metrics))

    def safe_add_tags(tags: Mapping[str, str]):
        span = ddtrace.tracer.current_span()
        if span:
            span.set_tags(cast(Any, tags))

    def safe_current_trace_context():  # pyright: ignore[reportRedeclaration]
        return ddtrace.tracer.current_trace_context()

    def safe_activate_trace_context(
        ctx: ddtrace.context.Context | ddtrace.Span | None,  # pyright: ignore[reportPrivateImportUsage]
    ) -> None:
        ddtrace.tracer.context_provider.activate(ctx)

else:

    def safe_set_gauge(gauge: str, value: int | float):
        pass

    def safe_incr(counter: str, value: int | float, tags: list[str] | None = None):
        pass

    @contextlib.contextmanager
    def safe_trace(span_id: str, attributes: Mapping[str, str] | None = None):
        yield

    def safe_add_metrics(metrics: Mapping[str, Union[int, float]]):
        pass

    def safe_add_tags(tags: Mapping[str, str]):
        pass

    def safe_current_trace_context():
        return

    def safe_activate_trace_context(
        ctx: ddtrace.context.Context | ddtrace.Span | None,  # pyright: ignore[reportPrivateImportUsage]
    ) -> None:
        pass

    def safe_distribution(counter: str, value: int | float, tags: list[str] | None = None):
        pass


class PerfTimer:
    def __init__(self):
        super().__init__()
        self._start = None
        self._end = None

    def __enter__(self):
        """Start a new timer as a context manager"""
        self._start = time.perf_counter()
        return self

    def __exit__(self, exc_typ: type[BaseException] | None, exc: BaseException | None, tb: types.TracebackType | None):
        """Stop the context manager timer"""
        self._end = time.perf_counter()

    @property
    def duration_seconds(self):
        assert self._start is not None
        end = time.perf_counter() if self._end is None else self._end
        return end - self._start

    @property
    def duration_ms(self):
        return self.duration_seconds * 1_000


def configure_tracing(default_service_name: str):
    from chalk.utils.log_with_context import get_logger

    _logger = get_logger(__name__)

    if not can_use_ddtrace:
        _logger.warning("ddtrace is not installed")
        return

    import ddtrace
    from ddtrace.filters import FilterRequestsOnUrl

    if ddtrace.config.service is None:
        ddtrace.config.service = default_service_name
    # Re-configuring the global tracer to capture any setting changes from environs from a .dotenv file
    # which might be loaded after the first ddtrace import

    ddtrace.tracer.configure(
        enabled=None if "DD_TRACE_ENABLED" not in os.environ else env_var_bool("DD_TRACE_ENABLED"),
        hostname=os.getenv("DD_AGENT_HOST") or os.getenv("DD_TRACE_AGENT_URL"),
        uds_path=os.getenv("DD_TRACE_AGENT_URL"),
        dogstatsd_url=os.getenv("DD_DOGSTATSD_URL"),
        api_version=os.getenv("DD_TRACE_API_VERSION"),
        compute_stats_enabled=env_var_bool("DD_TRACE_COMPUTE_STATS"),
        iast_enabled=None if "DD_IAST_ENABLED" not in os.environ else env_var_bool("DD_IAST_ENABLED"),
        # exclude healthcheck url from apm trace collection
        settings={
            "FILTERS": [
                FilterRequestsOnUrl(
                    [
                        r"^http://.*/healthcheck$",
                        r"^http://.*/ready$",
                        r"^http://[^/]*/$",  # exclude "/"
                    ]
                )
            ]
        },
    )
    if ddtrace.tracer.enabled:
        ddtrace.patch(
            asyncio=True,
            databricks=False,
            fastapi=True,
            futures=True,
            httplib=True,
            httpx=True,
            psycopg=True,
            redis=True,
            requests=True,
            sqlalchemy=False,
            urllib3=True,
        )

    _logger.info(
        f"Configuring DDtrace tracing: enabled={ddtrace.tracer.enabled}, service={ddtrace.config.service}, env={ddtrace.config.env}, trace_agent_url: {ddtrace.config._trace_agent_url}, effective trace agent: {ddtrace.tracer._agent_url}"  # pyright: ignore [reportAttributeAccessIssue, reportPrivateUsage]
    )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""

import grpc

from chalk._gen.chalk.aggregate.v1 import service_pb2 as chalk_dot_aggregate_dot_v1_dot_service__pb2
from chalk._gen.chalk.common.v1 import online_query_pb2 as chalk_dot_common_dot_v1_dot_online__query__pb2
from chalk._gen.chalk.common.v1 import upload_features_pb2 as chalk_dot_common_dot_v1_dot_upload__features__pb2
from chalk._gen.chalk.engine.v1 import query_server_pb2 as chalk_dot_engine_dot_v1_dot_query__server__pb2


class QueryServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Ping = channel.unary_unary(
            "/chalk.engine.v1.QueryService/Ping",
            request_serializer=chalk_dot_engine_dot_v1_dot_query__server__pb2.PingRequest.SerializeToString,
            response_deserializer=chalk_dot_engine_dot_v1_dot_query__server__pb2.PingResponse.FromString,
        )
        self.OnlineQuery = channel.unary_unary(
            "/chalk.engine.v1.QueryService/OnlineQuery",
            request_serializer=chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryRequest.SerializeToString,
            response_deserializer=chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryResponse.FromString,
        )
        self.OnlineQueryBulk = channel.unary_unary(
            "/chalk.engine.v1.QueryService/OnlineQueryBulk",
            request_serializer=chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryBulkRequest.SerializeToString,
            response_deserializer=chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryBulkResponse.FromString,
        )
        self.OnlineQueryMulti = channel.unary_unary(
            "/chalk.engine.v1.QueryService/OnlineQueryMulti",
            request_serializer=chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryMultiRequest.SerializeToString,
            response_deserializer=chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryMultiResponse.FromString,
        )
        self.UploadFeaturesBulk = channel.unary_unary(
            "/chalk.engine.v1.QueryService/UploadFeaturesBulk",
            request_serializer=chalk_dot_common_dot_v1_dot_online__query__pb2.UploadFeaturesBulkRequest.SerializeToString,
            response_deserializer=chalk_dot_common_dot_v1_dot_online__query__pb2.UploadFeaturesBulkResponse.FromString,
        )
        self.UploadFeatures = channel.unary_unary(
            "/chalk.engine.v1.QueryService/UploadFeatures",
            request_serializer=chalk_dot_common_dot_v1_dot_upload__features__pb2.UploadFeaturesRequest.SerializeToString,
            response_deserializer=chalk_dot_common_dot_v1_dot_upload__features__pb2.UploadFeaturesResponse.FromString,
        )
        self.PlanAggregateBackfill = channel.unary_unary(
            "/chalk.engine.v1.QueryService/PlanAggregateBackfill",
            request_serializer=chalk_dot_aggregate_dot_v1_dot_service__pb2.PlanAggregateBackfillRequest.SerializeToString,
            response_deserializer=chalk_dot_aggregate_dot_v1_dot_service__pb2.PlanAggregateBackfillResponse.FromString,
        )
        self.GetAggregates = channel.unary_unary(
            "/chalk.engine.v1.QueryService/GetAggregates",
            request_serializer=chalk_dot_aggregate_dot_v1_dot_service__pb2.GetAggregatesRequest.SerializeToString,
            response_deserializer=chalk_dot_aggregate_dot_v1_dot_service__pb2.GetAggregatesResponse.FromString,
        )


class QueryServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Ping(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def OnlineQuery(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def OnlineQueryBulk(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def OnlineQueryMulti(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UploadFeaturesBulk(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UploadFeatures(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def PlanAggregateBackfill(self, request, context):
        """PlanAggregateBackfill determines the estimated resources needed to backfill
        an aggregate.

        This method is a duplicate of the PlanAggregateBackfill method
        in the query_server.proto file. We should remove the query_server.proto method
        and move that request to this service instead.
        buf:lint:ignore RPC_REQUEST_RESPONSE_UNIQUE
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetAggregates(self, request, context):
        """This method is a duplicate of the PlanAggregateBackfill method
        in the query_server.proto file. We should remove the query_server.proto method
        and move that request to this service instead.
        buf:lint:ignore RPC_REQUEST_RESPONSE_UNIQUE
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_QueryServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Ping": grpc.unary_unary_rpc_method_handler(
            servicer.Ping,
            request_deserializer=chalk_dot_engine_dot_v1_dot_query__server__pb2.PingRequest.FromString,
            response_serializer=chalk_dot_engine_dot_v1_dot_query__server__pb2.PingResponse.SerializeToString,
        ),
        "OnlineQuery": grpc.unary_unary_rpc_method_handler(
            servicer.OnlineQuery,
            request_deserializer=chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryRequest.FromString,
            response_serializer=chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryResponse.SerializeToString,
        ),
        "OnlineQueryBulk": grpc.unary_unary_rpc_method_handler(
            servicer.OnlineQueryBulk,
            request_deserializer=chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryBulkRequest.FromString,
            response_serializer=chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryBulkResponse.SerializeToString,
        ),
        "OnlineQueryMulti": grpc.unary_unary_rpc_method_handler(
            servicer.OnlineQueryMulti,
            request_deserializer=chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryMultiRequest.FromString,
            response_serializer=chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryMultiResponse.SerializeToString,
        ),
        "UploadFeaturesBulk": grpc.unary_unary_rpc_method_handler(
            servicer.UploadFeaturesBulk,
            request_deserializer=chalk_dot_common_dot_v1_dot_online__query__pb2.UploadFeaturesBulkRequest.FromString,
            response_serializer=chalk_dot_common_dot_v1_dot_online__query__pb2.UploadFeaturesBulkResponse.SerializeToString,
        ),
        "UploadFeatures": grpc.unary_unary_rpc_method_handler(
            servicer.UploadFeatures,
            request_deserializer=chalk_dot_common_dot_v1_dot_upload__features__pb2.UploadFeaturesRequest.FromString,
            response_serializer=chalk_dot_common_dot_v1_dot_upload__features__pb2.UploadFeaturesResponse.SerializeToString,
        ),
        "PlanAggregateBackfill": grpc.unary_unary_rpc_method_handler(
            servicer.PlanAggregateBackfill,
            request_deserializer=chalk_dot_aggregate_dot_v1_dot_service__pb2.PlanAggregateBackfillRequest.FromString,
            response_serializer=chalk_dot_aggregate_dot_v1_dot_service__pb2.PlanAggregateBackfillResponse.SerializeToString,
        ),
        "GetAggregates": grpc.unary_unary_rpc_method_handler(
            servicer.GetAggregates,
            request_deserializer=chalk_dot_aggregate_dot_v1_dot_service__pb2.GetAggregatesRequest.FromString,
            response_serializer=chalk_dot_aggregate_dot_v1_dot_service__pb2.GetAggregatesResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("chalk.engine.v1.QueryService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class QueryService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Ping(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.engine.v1.QueryService/Ping",
            chalk_dot_engine_dot_v1_dot_query__server__pb2.PingRequest.SerializeToString,
            chalk_dot_engine_dot_v1_dot_query__server__pb2.PingResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def OnlineQuery(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.engine.v1.QueryService/OnlineQuery",
            chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryRequest.SerializeToString,
            chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def OnlineQueryBulk(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.engine.v1.QueryService/OnlineQueryBulk",
            chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryBulkRequest.SerializeToString,
            chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryBulkResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def OnlineQueryMulti(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.engine.v1.QueryService/OnlineQueryMulti",
            chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryMultiRequest.SerializeToString,
            chalk_dot_common_dot_v1_dot_online__query__pb2.OnlineQueryMultiResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UploadFeaturesBulk(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.engine.v1.QueryService/UploadFeaturesBulk",
            chalk_dot_common_dot_v1_dot_online__query__pb2.UploadFeaturesBulkRequest.SerializeToString,
            chalk_dot_common_dot_v1_dot_online__query__pb2.UploadFeaturesBulkResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UploadFeatures(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.engine.v1.QueryService/UploadFeatures",
            chalk_dot_common_dot_v1_dot_upload__features__pb2.UploadFeaturesRequest.SerializeToString,
            chalk_dot_common_dot_v1_dot_upload__features__pb2.UploadFeaturesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def PlanAggregateBackfill(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.engine.v1.QueryService/PlanAggregateBackfill",
            chalk_dot_aggregate_dot_v1_dot_service__pb2.PlanAggregateBackfillRequest.SerializeToString,
            chalk_dot_aggregate_dot_v1_dot_service__pb2.PlanAggregateBackfillResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetAggregates(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/chalk.engine.v1.QueryService/GetAggregates",
            chalk_dot_aggregate_dot_v1_dot_service__pb2.GetAggregatesRequest.SerializeToString,
            chalk_dot_aggregate_dot_v1_dot_service__pb2.GetAggregatesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



from __future__ import annotations

import dataclasses
import functools
import sys
from typing import Any

# Dataclasses has a bug on python 3.10 where the custom __getstate__ and __setstate__ is ignored if the class also defines __slots__
# See https://github.com/python/cpython/issues/104035
# Fixing via monkeypatch. The code below is pasted from
# https://github.com/python/cpython/blob/83518b3511712eb95e496bed5a8cf7498ebd02ea/Lib/dataclasses.py#L1122-L1146 and
# https://github.com/python/cpython/blob/72eea512b88f8fd68b7258242c37da963ad87360/Lib/dataclasses.py#L1223C1-L1228C51


def _add_slots(cls: type[Any], is_frozen: bool):
    # Need to create a new class, since we can't set __slots__
    #  after a class has been created.

    # Make sure __slots__ isn't already set.
    if "__slots__" in cls.__dict__:
        raise TypeError(f"{cls.__name__} already specifies __slots__")

    # Create a new dict for our new class.
    cls_dict = dict(cls.__dict__)
    field_names = tuple(f.name for f in dataclasses.fields(cls))
    cls_dict["__slots__"] = field_names
    for field_name in field_names:
        # Remove our attributes, if present. They'll still be
        #  available in _MARKER.
        cls_dict.pop(field_name, None)

    # Remove __dict__ itself.
    cls_dict.pop("__dict__", None)

    # And finally create the class.
    qualname = getattr(cls, "__qualname__", None)
    cls = type(cls)(cls.__name__, cls.__bases__, cls_dict)  # pyright: ignore
    if qualname is not None:
        cls.__qualname__ = qualname

    if is_frozen:
        # Need this for pickling frozen classes with slots.
        if "__getstate__" not in cls_dict:
            cls.__getstate__ = dataclasses._dataclass_getstate  # pyright: ignore
        if "__setstate__" not in cls_dict:
            cls.__setstate__ = dataclasses._dataclass_setstate  # pyright: ignore
    return cls


@functools.lru_cache(None)
def install_dataclass_slots_patch_if_needed():
    """Patch the implementation of slots when slots=True and frozen=True in python 3.10"""
    if sys.version_info < (3, 11):
        dataclasses._add_slots = _add_slots  # pyright: ignore -- we want to monkeypatch the buggy implementation



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/graph/v1/sources.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.arrow.v1 import arrow_pb2 as chalk_dot_arrow_dot_v1_dot_arrow__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1c\x63halk/graph/v1/sources.proto\x12\x0e\x63halk.graph.v1\x1a\x1a\x63halk/arrow/v1/arrow.proto\x1a\x1egoogle/protobuf/duration.proto"a\n\x15StreamSourceReference\x12\x34\n\x04type\x18\x01 \x01(\x0e\x32 .chalk.graph.v1.StreamSourceTypeR\x04type\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name"\xc0\x01\n\x0cStreamSource\x12\x33\n\x05kafka\x18\x01 \x01(\x0b\x32\x1b.chalk.graph.v1.KafkaSourceH\x00R\x05kafka\x12\x39\n\x07kinesis\x18\x02 \x01(\x0b\x32\x1d.chalk.graph.v1.KinesisSourceH\x00R\x07kinesis\x12\x36\n\x06pubsub\x18\x03 \x01(\x0b\x32\x1c.chalk.graph.v1.PubSubSourceH\x00R\x06pubsubB\x08\n\x06source"\xee\x03\n\rKinesisSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x1f\n\x0bstream_name\x18\x02 \x01(\tR\nstreamName\x12\x1d\n\nstream_arn\x18\x03 \x01(\tR\tstreamArn\x12\x1f\n\x0bregion_name\x18\x04 \x01(\tR\nregionName\x12M\n\x15late_arrival_deadline\x18\x05 \x01(\x0b\x32\x19.google.protobuf.DurationR\x13lateArrivalDeadline\x12@\n\x1d\x64\x65\x61\x64_letter_queue_stream_name\x18\x06 \x01(\tR\x19\x64\x65\x61\x64LetterQueueStreamName\x12)\n\x11\x61ws_access_key_id\x18\x07 \x01(\tR\x0e\x61wsAccessKeyId\x12\x31\n\x15\x61ws_secret_access_key\x18\x08 \x01(\tR\x12\x61wsSecretAccessKey\x12*\n\x11\x61ws_session_token\x18\t \x01(\tR\x0f\x61wsSessionToken\x12!\n\x0c\x65ndpoint_url\x18\n \x01(\tR\x0b\x65ndpointUrl\x12*\n\x11\x63onsumer_role_arn\x18\x0b \x01(\tR\x0f\x63onsumerRoleArn"\xae\x04\n\x0bKafkaSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12+\n\x11\x62ootstrap_servers\x18\x02 \x03(\tR\x10\x62ootstrapServers\x12\x14\n\x05topic\x18\x03 \x01(\tR\x05topic\x12\x32\n\x15ssl_keystore_location\x18\x04 \x01(\tR\x13sslKeystoreLocation\x12\x1e\n\x0bssl_ca_file\x18\x05 \x01(\tR\tsslCaFile\x12(\n\x10\x63lient_id_prefix\x18\x06 \x01(\tR\x0e\x63lientIdPrefix\x12&\n\x0fgroup_id_prefix\x18\x07 \x01(\tR\rgroupIdPrefix\x12+\n\x11security_protocol\x18\x08 \x01(\tR\x10securityProtocol\x12%\n\x0esasl_mechanism\x18\t \x01(\tR\rsaslMechanism\x12#\n\rsasl_username\x18\n \x01(\tR\x0csaslUsername\x12#\n\rsasl_password\x18\x0b \x01(\tR\x0csaslPassword\x12M\n\x15late_arrival_deadline\x18\x0c \x01(\x0b\x32\x19.google.protobuf.DurationR\x13lateArrivalDeadline\x12\x35\n\x17\x64\x65\x61\x64_letter_queue_topic\x18\r \x01(\tR\x14\x64\x65\x61\x64LetterQueueTopic"\xf0\x01\n\x0cPubSubSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x1d\n\nproject_id\x18\x02 \x01(\tR\tprojectId\x12\'\n\x0fsubscription_id\x18\x03 \x01(\tR\x0esubscriptionId\x12M\n\x15late_arrival_deadline\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationR\x13lateArrivalDeadline\x12\x35\n\x17\x64\x65\x61\x64_letter_queue_topic\x18\x05 \x01(\tR\x14\x64\x65\x61\x64LetterQueueTopic"e\n\x17\x44\x61tabaseSourceReference\x12\x36\n\x04type\x18\x01 \x01(\x0e\x32".chalk.graph.v1.DatabaseSourceTypeR\x04type\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name"\xae\x06\n\x0e\x44\x61tabaseSource\x12<\n\x08\x62igquery\x18\x01 \x01(\x0b\x32\x1e.chalk.graph.v1.BigQuerySourceH\x00R\x08\x62igquery\x12<\n\x08\x63loudsql\x18\x02 \x01(\x0b\x32\x1e.chalk.graph.v1.CloudSQLSourceH\x00R\x08\x63loudsql\x12\x42\n\ndatabricks\x18\x03 \x01(\x0b\x32 .chalk.graph.v1.DatabricksSourceH\x00R\ndatabricks\x12\x33\n\x05mysql\x18\x04 \x01(\x0b\x32\x1b.chalk.graph.v1.MySQLSourceH\x00R\x05mysql\x12<\n\x08postgres\x18\x05 \x01(\x0b\x32\x1e.chalk.graph.v1.PostgresSourceH\x00R\x08postgres\x12<\n\x08redshift\x18\x06 \x01(\x0b\x32\x1e.chalk.graph.v1.RedshiftSourceH\x00R\x08redshift\x12?\n\tsnowflake\x18\x07 \x01(\x0b\x32\x1f.chalk.graph.v1.SnowflakeSourceH\x00R\tsnowflake\x12\x36\n\x06sqlite\x18\x08 \x01(\x0b\x32\x1c.chalk.graph.v1.SQLiteSourceH\x00R\x06sqlite\x12\x39\n\x07spanner\x18\t \x01(\x0b\x32\x1d.chalk.graph.v1.SpannerSourceH\x00R\x07spanner\x12\x33\n\x05trino\x18\n \x01(\x0b\x32\x1b.chalk.graph.v1.TrinoSourceH\x00R\x05trino\x12<\n\x08\x64ynamodb\x18\x0b \x01(\x0b\x32\x1e.chalk.graph.v1.DynamoDBSourceH\x00R\x08\x64ynamodb\x12\x36\n\x06\x61thena\x18\x0c \x01(\x0b\x32\x1c.chalk.graph.v1.AthenaSourceH\x00R\x06\x61thena\x12\x42\n\nclickhouse\x18\r \x01(\x0b\x32 .chalk.graph.v1.ClickhouseSourceH\x00R\nclickhouseB\x08\n\x06source"\xbd\x04\n\x0e\x42igQuerySource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x18\n\x07project\x18\x02 \x01(\tR\x07project\x12\x18\n\x07\x64\x61taset\x18\x03 \x01(\tR\x07\x64\x61taset\x12\x1a\n\x08location\x18\x04 \x01(\tR\x08location\x12-\n\x12\x63redentials_base64\x18\x05 \x01(\tR\x11\x63redentialsBase64\x12)\n\x10\x63redentials_path\x18\x06 \x01(\tR\x0f\x63redentialsPath\x12O\n\x0b\x65ngine_args\x18\x07 \x03(\x0b\x32..chalk.graph.v1.BigQuerySource.EngineArgsEntryR\nengineArgs\x12_\n\x11\x61sync_engine_args\x18\x08 \x03(\x0b\x32\x33.chalk.graph.v1.BigQuerySource.AsyncEngineArgsEntryR\x0f\x61syncEngineArgs\x1aZ\n\x0f\x45ngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x1a_\n\x14\x41syncEngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01"\xf8\x03\n\x0e\x43loudSQLSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x0e\n\x02\x64\x62\x18\x02 \x01(\tR\x02\x64\x62\x12\x12\n\x04user\x18\x03 \x01(\tR\x04user\x12\x1a\n\x08password\x18\x04 \x01(\tR\x08password\x12#\n\rinstance_name\x18\x05 \x01(\tR\x0cinstanceName\x12O\n\x0b\x65ngine_args\x18\x06 \x03(\x0b\x32..chalk.graph.v1.CloudSQLSource.EngineArgsEntryR\nengineArgs\x12_\n\x11\x61sync_engine_args\x18\x07 \x03(\x0b\x32\x33.chalk.graph.v1.CloudSQLSource.AsyncEngineArgsEntryR\x0f\x61syncEngineArgs\x1aZ\n\x0f\x45ngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x1a_\n\x14\x41syncEngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01"\x91\x04\n\x10\x44\x61tabricksSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x12\n\x04host\x18\x02 \x01(\tR\x04host\x12\x12\n\x04port\x18\x03 \x01(\tR\x04port\x12\x0e\n\x02\x64\x62\x18\x04 \x01(\tR\x02\x64\x62\x12\x1b\n\thttp_path\x18\x05 \x01(\tR\x08httpPath\x12!\n\x0c\x61\x63\x63\x65ss_token\x18\x06 \x01(\tR\x0b\x61\x63\x63\x65ssToken\x12Q\n\x0b\x65ngine_args\x18\x07 \x03(\x0b\x32\x30.chalk.graph.v1.DatabricksSource.EngineArgsEntryR\nengineArgs\x12\x61\n\x11\x61sync_engine_args\x18\x08 \x03(\x0b\x32\x35.chalk.graph.v1.DatabricksSource.AsyncEngineArgsEntryR\x0f\x61syncEngineArgs\x1aZ\n\x0f\x45ngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x1a_\n\x14\x41syncEngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01"\xde\x05\n\x0e\x44ynamoDBSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x38\n\x16\x61ws_client_id_override\x18\x02 \x01(\tH\x00R\x13\x61wsClientIdOverride\x88\x01\x01\x12@\n\x1a\x61ws_client_secret_override\x18\x03 \x01(\tH\x01R\x17\x61wsClientSecretOverride\x88\x01\x01\x12\x33\n\x13\x61ws_region_override\x18\x04 \x01(\tH\x02R\x11\x61wsRegionOverride\x88\x01\x01\x12\x30\n\x11\x65ndpoint_override\x18\x05 \x01(\tH\x03R\x10\x65ndpointOverride\x88\x01\x01\x12O\n\x0b\x65ngine_args\x18\x06 \x03(\x0b\x32..chalk.graph.v1.DynamoDBSource.EngineArgsEntryR\nengineArgs\x12_\n\x11\x61sync_engine_args\x18\x07 \x03(\x0b\x32\x33.chalk.graph.v1.DynamoDBSource.AsyncEngineArgsEntryR\x0f\x61syncEngineArgs\x1aZ\n\x0f\x45ngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x1a_\n\x14\x41syncEngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x42\x19\n\x17_aws_client_id_overrideB\x1d\n\x1b_aws_client_secret_overrideB\x16\n\x14_aws_region_overrideB\x14\n\x12_endpoint_override"\xf2\x03\n\x0bMySQLSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x12\n\x04host\x18\x02 \x01(\tR\x04host\x12\x12\n\x04port\x18\x03 \x01(\tR\x04port\x12\x0e\n\x02\x64\x62\x18\x04 \x01(\tR\x02\x64\x62\x12\x12\n\x04user\x18\x05 \x01(\tR\x04user\x12\x1a\n\x08password\x18\x06 \x01(\tR\x08password\x12L\n\x0b\x65ngine_args\x18\x07 \x03(\x0b\x32+.chalk.graph.v1.MySQLSource.EngineArgsEntryR\nengineArgs\x12\\\n\x11\x61sync_engine_args\x18\x08 \x03(\x0b\x32\x30.chalk.graph.v1.MySQLSource.AsyncEngineArgsEntryR\x0f\x61syncEngineArgs\x1aZ\n\x0f\x45ngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x1a_\n\x14\x41syncEngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01"\xfb\x03\n\x0ePostgresSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x12\n\x04host\x18\x02 \x01(\tR\x04host\x12\x12\n\x04port\x18\x03 \x01(\tR\x04port\x12\x0e\n\x02\x64\x62\x18\x04 \x01(\tR\x02\x64\x62\x12\x12\n\x04user\x18\x05 \x01(\tR\x04user\x12\x1a\n\x08password\x18\x06 \x01(\tR\x08password\x12O\n\x0b\x65ngine_args\x18\x07 \x03(\x0b\x32..chalk.graph.v1.PostgresSource.EngineArgsEntryR\nengineArgs\x12_\n\x11\x61sync_engine_args\x18\x08 \x03(\x0b\x32\x33.chalk.graph.v1.PostgresSource.AsyncEngineArgsEntryR\x0f\x61syncEngineArgs\x1aZ\n\x0f\x45ngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x1a_\n\x14\x41syncEngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01"\xdd\x04\n\x0eRedshiftSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x12\n\x04host\x18\x02 \x01(\tR\x04host\x12\x12\n\x04port\x18\x03 \x01(\tR\x04port\x12\x0e\n\x02\x64\x62\x18\x04 \x01(\tR\x02\x64\x62\x12\x12\n\x04user\x18\x05 \x01(\tR\x04user\x12\x1a\n\x08password\x18\x06 \x01(\tR\x08password\x12\x1b\n\ts3_client\x18\x07 \x01(\tR\x08s3Client\x12\x1b\n\ts3_bucket\x18\x08 \x01(\tR\x08s3Bucket\x12O\n\x0b\x65ngine_args\x18\t \x03(\x0b\x32..chalk.graph.v1.RedshiftSource.EngineArgsEntryR\nengineArgs\x12_\n\x11\x61sync_engine_args\x18\n \x03(\x0b\x32\x33.chalk.graph.v1.RedshiftSource.AsyncEngineArgsEntryR\x0f\x61syncEngineArgs\x12&\n\x0funload_iam_role\x18\x0b \x01(\tR\runloadIamRole\x1aZ\n\x0f\x45ngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x1a_\n\x14\x41syncEngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01"\xf7\x04\n\x0fSnowflakeSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x0e\n\x02\x64\x62\x18\x02 \x01(\tR\x02\x64\x62\x12\x16\n\x06schema\x18\x03 \x01(\tR\x06schema\x12\x12\n\x04role\x18\x04 \x01(\tR\x04role\x12\x12\n\x04user\x18\x05 \x01(\tR\x04user\x12\x1a\n\x08password\x18\x06 \x01(\tR\x08password\x12-\n\x12\x61\x63\x63ount_identifier\x18\x07 \x01(\tR\x11\x61\x63\x63ountIdentifier\x12\x1c\n\twarehouse\x18\x08 \x01(\tR\twarehouse\x12P\n\x0b\x65ngine_args\x18\t \x03(\x0b\x32/.chalk.graph.v1.SnowflakeSource.EngineArgsEntryR\nengineArgs\x12`\n\x11\x61sync_engine_args\x18\n \x03(\x0b\x32\x34.chalk.graph.v1.SnowflakeSource.AsyncEngineArgsEntryR\x0f\x61syncEngineArgs\x12&\n\x0fprivate_key_b64\x18\x0b \x01(\tR\rprivateKeyB64\x1aZ\n\x0f\x45ngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x1a_\n\x14\x41syncEngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01"\xaa\x03\n\x0cSQLiteSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x1b\n\tfile_name\x18\x02 \x01(\tR\x08\x66ileName\x12M\n\x0b\x65ngine_args\x18\x03 \x03(\x0b\x32,.chalk.graph.v1.SQLiteSource.EngineArgsEntryR\nengineArgs\x12]\n\x11\x61sync_engine_args\x18\x04 \x03(\x0b\x32\x31.chalk.graph.v1.SQLiteSource.AsyncEngineArgsEntryR\x0f\x61syncEngineArgs\x1aZ\n\x0f\x45ngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x1a_\n\x14\x41syncEngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01"\x85\x04\n\rSpannerSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x18\n\x07project\x18\x02 \x01(\tR\x07project\x12\x1a\n\x08instance\x18\x03 \x01(\tR\x08instance\x12\x0e\n\x02\x64\x62\x18\x04 \x01(\tR\x02\x64\x62\x12-\n\x12\x63redentials_base64\x18\x05 \x01(\tR\x11\x63redentialsBase64\x12N\n\x0b\x65ngine_args\x18\x06 \x03(\x0b\x32-.chalk.graph.v1.SpannerSource.EngineArgsEntryR\nengineArgs\x12^\n\x11\x61sync_engine_args\x18\x07 \x03(\x0b\x32\x32.chalk.graph.v1.SpannerSource.AsyncEngineArgsEntryR\x0f\x61syncEngineArgs\x1aZ\n\x0f\x45ngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x1a_\n\x14\x41syncEngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01"\x94\x04\n\x0bTrinoSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x12\n\x04host\x18\x02 \x01(\tR\x04host\x12\x12\n\x04port\x18\x03 \x01(\tR\x04port\x12\x18\n\x07\x63\x61talog\x18\x04 \x01(\tR\x07\x63\x61talog\x12\x16\n\x06schema\x18\x05 \x01(\tR\x06schema\x12\x12\n\x04user\x18\x06 \x01(\tR\x04user\x12\x1a\n\x08password\x18\x07 \x01(\tR\x08password\x12L\n\x0b\x65ngine_args\x18\x08 \x03(\x0b\x32+.chalk.graph.v1.TrinoSource.EngineArgsEntryR\nengineArgs\x12\\\n\x11\x61sync_engine_args\x18\t \x03(\x0b\x32\x30.chalk.graph.v1.TrinoSource.AsyncEngineArgsEntryR\x0f\x61syncEngineArgs\x1aZ\n\x0f\x45ngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x1a_\n\x14\x41syncEngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01"\xb9\x06\n\x0c\x41thenaSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x1d\n\naws_region\x18\x02 \x01(\tR\tawsRegion\x12.\n\x11\x61ws_access_key_id\x18\x03 \x01(\tH\x00R\x0e\x61wsAccessKeyId\x88\x01\x01\x12\x36\n\x15\x61ws_access_key_secret\x18\x04 \x01(\tH\x01R\x12\x61wsAccessKeySecret\x88\x01\x01\x12$\n\x0es3_staging_dir\x18\x05 \x01(\tR\x0cs3StagingDir\x12&\n\x0c\x63\x61talog_name\x18\x06 \x01(\tH\x02R\x0b\x63\x61talogName\x88\x01\x01\x12$\n\x0bschema_name\x18\x07 \x01(\tH\x03R\nschemaName\x88\x01\x01\x12\x1e\n\x08role_arn\x18\x08 \x01(\tH\x04R\x07roleArn\x88\x01\x01\x12M\n\x0b\x65ngine_args\x18\t \x03(\x0b\x32,.chalk.graph.v1.AthenaSource.EngineArgsEntryR\nengineArgs\x12]\n\x11\x61sync_engine_args\x18\n \x03(\x0b\x32\x31.chalk.graph.v1.AthenaSource.AsyncEngineArgsEntryR\x0f\x61syncEngineArgs\x12"\n\nwork_group\x18\x0b \x01(\tH\x05R\tworkGroup\x88\x01\x01\x1aZ\n\x0f\x45ngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x1a_\n\x14\x41syncEngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x42\x14\n\x12_aws_access_key_idB\x18\n\x16_aws_access_key_secretB\x0f\n\r_catalog_nameB\x0e\n\x0c_schema_nameB\x0b\n\t_role_arnB\r\n\x0b_work_group"\x9a\x04\n\x10\x43lickhouseSource\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x12\n\x04host\x18\x02 \x01(\tR\x04host\x12\x12\n\x04port\x18\x03 \x01(\tR\x04port\x12\x0e\n\x02\x64\x62\x18\x04 \x01(\tR\x02\x64\x62\x12\x12\n\x04user\x18\x05 \x01(\tR\x04user\x12\x1a\n\x08password\x18\x06 \x01(\tR\x08password\x12\x17\n\x07use_tls\x18\x07 \x01(\x08R\x06useTls\x12Q\n\x0b\x65ngine_args\x18\x08 \x03(\x0b\x32\x30.chalk.graph.v1.ClickhouseSource.EngineArgsEntryR\nengineArgs\x12\x61\n\x11\x61sync_engine_args\x18\t \x03(\x0b\x32\x35.chalk.graph.v1.ClickhouseSource.AsyncEngineArgsEntryR\x0f\x61syncEngineArgs\x1aZ\n\x0f\x45ngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01\x1a_\n\x14\x41syncEngineArgsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x05value:\x02\x38\x01*\x93\x01\n\x10StreamSourceType\x12"\n\x1eSTREAM_SOURCE_TYPE_UNSPECIFIED\x10\x00\x12\x1c\n\x18STREAM_SOURCE_TYPE_KAFKA\x10\x01\x12\x1e\n\x1aSTREAM_SOURCE_TYPE_KINESIS\x10\x02\x12\x1d\n\x19STREAM_SOURCE_TYPE_PUBSUB\x10\x03*\xd6\x03\n\x12\x44\x61tabaseSourceType\x12$\n DATABASE_SOURCE_TYPE_UNSPECIFIED\x10\x00\x12!\n\x1d\x44\x41TABASE_SOURCE_TYPE_BIGQUERY\x10\x01\x12!\n\x1d\x44\x41TABASE_SOURCE_TYPE_CLOUDSQL\x10\x02\x12#\n\x1f\x44\x41TABASE_SOURCE_TYPE_DATABRICKS\x10\x03\x12\x1e\n\x1a\x44\x41TABASE_SOURCE_TYPE_MYSQL\x10\x04\x12!\n\x1d\x44\x41TABASE_SOURCE_TYPE_POSTGRES\x10\x05\x12!\n\x1d\x44\x41TABASE_SOURCE_TYPE_REDSHIFT\x10\x06\x12"\n\x1e\x44\x41TABASE_SOURCE_TYPE_SNOWFLAKE\x10\x07\x12\x1f\n\x1b\x44\x41TABASE_SOURCE_TYPE_SQLITE\x10\x08\x12 \n\x1c\x44\x41TABASE_SOURCE_TYPE_SPANNER\x10\t\x12\x1e\n\x1a\x44\x41TABASE_SOURCE_TYPE_TRINO\x10\n\x12!\n\x1d\x44\x41TABASE_SOURCE_TYPE_DYNAMODB\x10\x0b\x12\x1f\n\x1b\x44\x41TABASE_SOURCE_TYPE_ATHENA\x10\x0c\x42|\n\x12\x63om.chalk.graph.v1B\x0cSourcesProtoP\x01\xa2\x02\x03\x43GX\xaa\x02\x0e\x43halk.Graph.V1\xca\x02\x0e\x43halk\\Graph\\V1\xe2\x02\x1a\x43halk\\Graph\\V1\\GPBMetadata\xea\x02\x10\x43halk::Graph::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.graph.v1.sources_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\022com.chalk.graph.v1B\014SourcesProtoP\001\242\002\003CGX\252\002\016Chalk.Graph.V1\312\002\016Chalk\\Graph\\V1\342\002\032Chalk\\Graph\\V1\\GPBMetadata\352\002\020Chalk::Graph::V1"
    _globals["_BIGQUERYSOURCE_ENGINEARGSENTRY"]._options = None
    _globals["_BIGQUERYSOURCE_ENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_BIGQUERYSOURCE_ASYNCENGINEARGSENTRY"]._options = None
    _globals["_BIGQUERYSOURCE_ASYNCENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_CLOUDSQLSOURCE_ENGINEARGSENTRY"]._options = None
    _globals["_CLOUDSQLSOURCE_ENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_CLOUDSQLSOURCE_ASYNCENGINEARGSENTRY"]._options = None
    _globals["_CLOUDSQLSOURCE_ASYNCENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_DATABRICKSSOURCE_ENGINEARGSENTRY"]._options = None
    _globals["_DATABRICKSSOURCE_ENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_DATABRICKSSOURCE_ASYNCENGINEARGSENTRY"]._options = None
    _globals["_DATABRICKSSOURCE_ASYNCENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_DYNAMODBSOURCE_ENGINEARGSENTRY"]._options = None
    _globals["_DYNAMODBSOURCE_ENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_DYNAMODBSOURCE_ASYNCENGINEARGSENTRY"]._options = None
    _globals["_DYNAMODBSOURCE_ASYNCENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_MYSQLSOURCE_ENGINEARGSENTRY"]._options = None
    _globals["_MYSQLSOURCE_ENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_MYSQLSOURCE_ASYNCENGINEARGSENTRY"]._options = None
    _globals["_MYSQLSOURCE_ASYNCENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_POSTGRESSOURCE_ENGINEARGSENTRY"]._options = None
    _globals["_POSTGRESSOURCE_ENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_POSTGRESSOURCE_ASYNCENGINEARGSENTRY"]._options = None
    _globals["_POSTGRESSOURCE_ASYNCENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_REDSHIFTSOURCE_ENGINEARGSENTRY"]._options = None
    _globals["_REDSHIFTSOURCE_ENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_REDSHIFTSOURCE_ASYNCENGINEARGSENTRY"]._options = None
    _globals["_REDSHIFTSOURCE_ASYNCENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_SNOWFLAKESOURCE_ENGINEARGSENTRY"]._options = None
    _globals["_SNOWFLAKESOURCE_ENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_SNOWFLAKESOURCE_ASYNCENGINEARGSENTRY"]._options = None
    _globals["_SNOWFLAKESOURCE_ASYNCENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_SQLITESOURCE_ENGINEARGSENTRY"]._options = None
    _globals["_SQLITESOURCE_ENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_SQLITESOURCE_ASYNCENGINEARGSENTRY"]._options = None
    _globals["_SQLITESOURCE_ASYNCENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_SPANNERSOURCE_ENGINEARGSENTRY"]._options = None
    _globals["_SPANNERSOURCE_ENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_SPANNERSOURCE_ASYNCENGINEARGSENTRY"]._options = None
    _globals["_SPANNERSOURCE_ASYNCENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_TRINOSOURCE_ENGINEARGSENTRY"]._options = None
    _globals["_TRINOSOURCE_ENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_TRINOSOURCE_ASYNCENGINEARGSENTRY"]._options = None
    _globals["_TRINOSOURCE_ASYNCENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_ATHENASOURCE_ENGINEARGSENTRY"]._options = None
    _globals["_ATHENASOURCE_ENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_ATHENASOURCE_ASYNCENGINEARGSENTRY"]._options = None
    _globals["_ATHENASOURCE_ASYNCENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_CLICKHOUSESOURCE_ENGINEARGSENTRY"]._options = None
    _globals["_CLICKHOUSESOURCE_ENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_CLICKHOUSESOURCE_ASYNCENGINEARGSENTRY"]._options = None
    _globals["_CLICKHOUSESOURCE_ASYNCENGINEARGSENTRY"]._serialized_options = b"8\001"
    _globals["_STREAMSOURCETYPE"]._serialized_start = 10082
    _globals["_STREAMSOURCETYPE"]._serialized_end = 10229
    _globals["_DATABASESOURCETYPE"]._serialized_start = 10232
    _globals["_DATABASESOURCETYPE"]._serialized_end = 10702
    _globals["_STREAMSOURCEREFERENCE"]._serialized_start = 108
    _globals["_STREAMSOURCEREFERENCE"]._serialized_end = 205
    _globals["_STREAMSOURCE"]._serialized_start = 208
    _globals["_STREAMSOURCE"]._serialized_end = 400
    _globals["_KINESISSOURCE"]._serialized_start = 403
    _globals["_KINESISSOURCE"]._serialized_end = 897
    _globals["_KAFKASOURCE"]._serialized_start = 900
    _globals["_KAFKASOURCE"]._serialized_end = 1458
    _globals["_PUBSUBSOURCE"]._serialized_start = 1461
    _globals["_PUBSUBSOURCE"]._serialized_end = 1701
    _globals["_DATABASESOURCEREFERENCE"]._serialized_start = 1703
    _globals["_DATABASESOURCEREFERENCE"]._serialized_end = 1804
    _globals["_DATABASESOURCE"]._serialized_start = 1807
    _globals["_DATABASESOURCE"]._serialized_end = 2621
    _globals["_BIGQUERYSOURCE"]._serialized_start = 2624
    _globals["_BIGQUERYSOURCE"]._serialized_end = 3197
    _globals["_BIGQUERYSOURCE_ENGINEARGSENTRY"]._serialized_start = 3010
    _globals["_BIGQUERYSOURCE_ENGINEARGSENTRY"]._serialized_end = 3100
    _globals["_BIGQUERYSOURCE_ASYNCENGINEARGSENTRY"]._serialized_start = 3102
    _globals["_BIGQUERYSOURCE_ASYNCENGINEARGSENTRY"]._serialized_end = 3197
    _globals["_CLOUDSQLSOURCE"]._serialized_start = 3200
    _globals["_CLOUDSQLSOURCE"]._serialized_end = 3704
    _globals["_CLOUDSQLSOURCE_ENGINEARGSENTRY"]._serialized_start = 3010
    _globals["_CLOUDSQLSOURCE_ENGINEARGSENTRY"]._serialized_end = 3100
    _globals["_CLOUDSQLSOURCE_ASYNCENGINEARGSENTRY"]._serialized_start = 3102
    _globals["_CLOUDSQLSOURCE_ASYNCENGINEARGSENTRY"]._serialized_end = 3197
    _globals["_DATABRICKSSOURCE"]._serialized_start = 3707
    _globals["_DATABRICKSSOURCE"]._serialized_end = 4236
    _globals["_DATABRICKSSOURCE_ENGINEARGSENTRY"]._serialized_start = 3010
    _globals["_DATABRICKSSOURCE_ENGINEARGSENTRY"]._serialized_end = 3100
    _globals["_DATABRICKSSOURCE_ASYNCENGINEARGSENTRY"]._serialized_start = 3102
    _globals["_DATABRICKSSOURCE_ASYNCENGINEARGSENTRY"]._serialized_end = 3197
    _globals["_DYNAMODBSOURCE"]._serialized_start = 4239
    _globals["_DYNAMODBSOURCE"]._serialized_end = 4973
    _globals["_DYNAMODBSOURCE_ENGINEARGSENTRY"]._serialized_start = 3010
    _globals["_DYNAMODBSOURCE_ENGINEARGSENTRY"]._serialized_end = 3100
    _globals["_DYNAMODBSOURCE_ASYNCENGINEARGSENTRY"]._serialized_start = 3102
    _globals["_DYNAMODBSOURCE_ASYNCENGINEARGSENTRY"]._serialized_end = 3197
    _globals["_MYSQLSOURCE"]._serialized_start = 4976
    _globals["_MYSQLSOURCE"]._serialized_end = 5474
    _globals["_MYSQLSOURCE_ENGINEARGSENTRY"]._serialized_start = 3010
    _globals["_MYSQLSOURCE_ENGINEARGSENTRY"]._serialized_end = 3100
    _globals["_MYSQLSOURCE_ASYNCENGINEARGSENTRY"]._serialized_start = 3102
    _globals["_MYSQLSOURCE_ASYNCENGINEARGSENTRY"]._serialized_end = 3197
    _globals["_POSTGRESSOURCE"]._serialized_start = 5477
    _globals["_POSTGRESSOURCE"]._serialized_end = 5984
    _globals["_POSTGRESSOURCE_ENGINEARGSENTRY"]._serialized_start = 3010
    _globals["_POSTGRESSOURCE_ENGINEARGSENTRY"]._serialized_end = 3100
    _globals["_POSTGRESSOURCE_ASYNCENGINEARGSENTRY"]._serialized_start = 3102
    _globals["_POSTGRESSOURCE_ASYNCENGINEARGSENTRY"]._serialized_end = 3197
    _globals["_REDSHIFTSOURCE"]._serialized_start = 5987
    _globals["_REDSHIFTSOURCE"]._serialized_end = 6592
    _globals["_REDSHIFTSOURCE_ENGINEARGSENTRY"]._serialized_start = 3010
    _globals["_REDSHIFTSOURCE_ENGINEARGSENTRY"]._serialized_end = 3100
    _globals["_REDSHIFTSOURCE_ASYNCENGINEARGSENTRY"]._serialized_start = 3102
    _globals["_REDSHIFTSOURCE_ASYNCENGINEARGSENTRY"]._serialized_end = 3197
    _globals["_SNOWFLAKESOURCE"]._serialized_start = 6595
    _globals["_SNOWFLAKESOURCE"]._serialized_end = 7226
    _globals["_SNOWFLAKESOURCE_ENGINEARGSENTRY"]._serialized_start = 3010
    _globals["_SNOWFLAKESOURCE_ENGINEARGSENTRY"]._serialized_end = 3100
    _globals["_SNOWFLAKESOURCE_ASYNCENGINEARGSENTRY"]._serialized_start = 3102
    _globals["_SNOWFLAKESOURCE_ASYNCENGINEARGSENTRY"]._serialized_end = 3197
    _globals["_SQLITESOURCE"]._serialized_start = 7229
    _globals["_SQLITESOURCE"]._serialized_end = 7655
    _globals["_SQLITESOURCE_ENGINEARGSENTRY"]._serialized_start = 3010
    _globals["_SQLITESOURCE_ENGINEARGSENTRY"]._serialized_end = 3100
    _globals["_SQLITESOURCE_ASYNCENGINEARGSENTRY"]._serialized_start = 3102
    _globals["_SQLITESOURCE_ASYNCENGINEARGSENTRY"]._serialized_end = 3197
    _globals["_SPANNERSOURCE"]._serialized_start = 7658
    _globals["_SPANNERSOURCE"]._serialized_end = 8175
    _globals["_SPANNERSOURCE_ENGINEARGSENTRY"]._serialized_start = 3010
    _globals["_SPANNERSOURCE_ENGINEARGSENTRY"]._serialized_end = 3100
    _globals["_SPANNERSOURCE_ASYNCENGINEARGSENTRY"]._serialized_start = 3102
    _globals["_SPANNERSOURCE_ASYNCENGINEARGSENTRY"]._serialized_end = 3197
    _globals["_TRINOSOURCE"]._serialized_start = 8178
    _globals["_TRINOSOURCE"]._serialized_end = 8710
    _globals["_TRINOSOURCE_ENGINEARGSENTRY"]._serialized_start = 3010
    _globals["_TRINOSOURCE_ENGINEARGSENTRY"]._serialized_end = 3100
    _globals["_TRINOSOURCE_ASYNCENGINEARGSENTRY"]._serialized_start = 3102
    _globals["_TRINOSOURCE_ASYNCENGINEARGSENTRY"]._serialized_end = 3197
    _globals["_ATHENASOURCE"]._serialized_start = 8713
    _globals["_ATHENASOURCE"]._serialized_end = 9538
    _globals["_ATHENASOURCE_ENGINEARGSENTRY"]._serialized_start = 3010
    _globals["_ATHENASOURCE_ENGINEARGSENTRY"]._serialized_end = 3100
    _globals["_ATHENASOURCE_ASYNCENGINEARGSENTRY"]._serialized_start = 3102
    _globals["_ATHENASOURCE_ASYNCENGINEARGSENTRY"]._serialized_end = 3197
    _globals["_CLICKHOUSESOURCE"]._serialized_start = 9541
    _globals["_CLICKHOUSESOURCE"]._serialized_end = 10079
    _globals["_CLICKHOUSESOURCE_ENGINEARGSENTRY"]._serialized_start = 3010
    _globals["_CLICKHOUSESOURCE_ENGINEARGSENTRY"]._serialized_end = 3100
    _globals["_CLICKHOUSESOURCE_ASYNCENGINEARGSENTRY"]._serialized_start = 3102
    _globals["_CLICKHOUSESOURCE_ASYNCENGINEARGSENTRY"]._serialized_end = 3197
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

import datetime
import functools
from typing import Any

import orjson
import sqlalchemy as sa
from snowflake.sqlalchemy import ARRAY, OBJECT
from snowflake.sqlalchemy.base import SnowflakeCompiler, SnowflakeDDLCompiler, SnowflakeTypeCompiler
from snowflake.sqlalchemy.snowdialect import TIMESTAMP_LTZ, TIMESTAMP_NTZ, TIMESTAMP_TZ, SnowflakeDialect
from sqlalchemy.engine import Dialect
from sqlalchemy.ext.compiler import compiles
from sqlalchemy.schema import CreateIndex, DropIndex
from sqlalchemy.sql import functions  # noqa # pyright thinks this doesn't exist
from sqlalchemy.sql.functions import AnsiFunction
from sqlalchemy.sql.sqltypes import DATETIME, TIMESTAMP
from sqlalchemy.sql.type_api import NativeForEmulated
from sqlalchemy.types import TypeEngine

from chalk.utils.collections import get_unique_item


def _datetime_bind_processor(self: TypeEngine, dialect: Dialect):
    def _processor(val: datetime.datetime | None):
        if val is None:
            return None
        return _format_datetime_as_iso(val)

    return _processor


class IntervalType(NativeForEmulated, TypeEngine):
    __visit_name__ = "INTERVAL"
    native = True

    @property
    def python_type(self):
        return datetime.timedelta

    def compile(self, dialect: Dialect | None = None):
        return "INTERVAL"

    def as_generic(self, allow_nulltype: bool = True) -> TypeEngine[Any]:
        return sa.Interval(native=True)

    def bind_expression(self, bindvalue: Any) -> Any:
        return sa.func.interval(bindvalue)

    def bind_processor(self, dialect: Dialect):
        def _processor(val: datetime.timedelta | None):
            assert isinstance(val, datetime.timedelta), "intervals must be timedeltas"
            if val < datetime.timedelta(0):
                raise ValueError(
                    (
                        f"Interval '{val}' is negative, which is not supported on snowflake. "
                        "Instead, please modify the query to subtract a positive internal instead of adding a negative interval."
                    )
                )
            parts: list[str] = []
            parts.append(f"{val.days} d")
            if val.seconds > 0:
                parts.append(f"{val.seconds} s")
            if val.microseconds > 0:
                parts.append(f"{val.microseconds} us")
            return ", ".join(parts)

        return _processor


class interval(AnsiFunction):
    inherit_cache = True


def _compile_interval_func(element: interval, compiler: SnowflakeCompiler, **kw: Any):
    clause = get_unique_item(element.clauses.clauses)
    return f"interval {compiler.process(clause, **kw)}"


def _compile_interval(element: IntervalType, compiler: SnowflakeTypeCompiler, **kw: Any):
    return "INTERVAL"


def _compile_func_now(element: functions.now, compiler: SnowflakeCompiler, **kw: Any):
    return "CURRENT_TIMESTAMP()"


def _compile_create_index(element: CreateIndex, compiler: SnowflakeDDLCompiler, **kw: Any):
    # Snowflake does not support indexing
    return "SELECT 1"


def _compile_drop_index(element: DropIndex, compiler: SnowflakeDDLCompiler, **kw: Any):
    # Snowflake does not support indexing
    return "SELECT 1"


def _compile_datetime(element: sa.DateTime | sa.TIMESTAMP | sa.DATETIME, compiler: SnowflakeTypeCompiler, **kw: Any):
    if element.timezone:
        return "TIMESTAMP_TZ"
    else:
        return "TIMESTAMP_NTZ"


class ArrayType(ARRAY):
    def get_dbapi_type(self, dbapi: Any):
        return dbapi.ARRAY

    def bind_processor(self, dialect: Dialect, **kw: Any):
        # Convert any python values into a json string when serializing
        # Using orjson instead of json for performance and to coerce nan/+inf/-inf to null values
        return lambda x: orjson.dumps(x).decode("utf8")

    def result_processor(self, dialect: Dialect, coltype: Any):
        def _result_processor(val: str | None):
            if val is None:
                return None
            return orjson.loads(val)

        return _result_processor


class ObjectType(OBJECT):
    def get_dbapi_type(self, dbapi: Any):
        return dbapi.OBJECT

    def bind_processor(self, dialect: Dialect, **kw: Any):
        # Convert any python values into a json string when serializing
        # Using orjson instead of json for performance and to coerce nan/+inf/-inf to null values
        return lambda x: orjson.dumps(x).decode("utf8")

    def result_processor(self, dialect: Dialect, coltype: Any):
        def _result_processor(val: str | None):
            if val is None:
                return None
            return orjson.loads(val)

        return _result_processor


def _format_datetime_as_iso(val: datetime.datetime):
    """Format a datetime as standardized ISO string, independent of platform"""
    # See https://bugs.python.org/issue13305 -- `datetime.datetime.min.isoformat()` is incorrect on linux
    # It returns 1-01-01T00:00:00 instead of 0001-01-01T00:00:00
    time_format = val.time().isoformat()
    return f"{val.year:04d}-{val.month:02d}-{val.day:02d}T{time_format}"


@functools.lru_cache(None)
def register_snowflake_compiler_hooks():
    colspecs = dict(SnowflakeDialect.colspecs)
    colspecs[sa.ARRAY] = ArrayType
    colspecs[ARRAY] = ArrayType
    colspecs[sa.Interval] = IntervalType
    colspecs[DATETIME] = TIMESTAMP_NTZ
    colspecs[TIMESTAMP] = TIMESTAMP_TZ
    TIMESTAMP_NTZ.bind_processor = _datetime_bind_processor
    TIMESTAMP_TZ.bind_processor = _datetime_bind_processor
    TIMESTAMP_LTZ.bind_processor = _datetime_bind_processor
    colspecs[OBJECT] = ObjectType
    SnowflakeDialect.colspecs = colspecs
    compiles(CreateIndex, "snowflake")(_compile_create_index)
    compiles(DropIndex, "snowflake")(_compile_drop_index)
    compiles(IntervalType, "snowflake")(_compile_interval)
    compiles(sa.DateTime, "snowflake")(_compile_datetime)
    compiles(sa.TIMESTAMP, "snowflake")(_compile_datetime)
    compiles(sa.DATETIME, "snowflake")(_compile_datetime)
    compiles(interval, "snowflake")(_compile_interval_func)
    compiles(functions.now, "snowflake")(_compile_func_now)



from __future__ import annotations

from chalk._validation.validation import Validation
from chalk._version import __version__
from chalk.features import (
    Cron,
    DataFrame,
    Environments,
    Features,
    FeatureTime,
    Primary,
    Tags,
    after,
    before,
    description,
    embed,
    feature,
    has_many,
    has_one,
    is_primary,
    op,
    owner,
    tags,
)
from chalk.features._document import Document
from chalk.features._last import Last
from chalk.features.filter import freeze_time
from chalk.features.pseudofeatures import Distance, Now
from chalk.features.resolver import OfflineResolver, OnlineResolver, Resolver, offline, online
from chalk.features.tag import BranchId, EnvironmentId
from chalk.features.underscore import _, __, underscore
from chalk.importer import get_resolver
from chalk.logging import chalk_logger
from chalk.operators import StaticOperator, scan_parquet
from chalk.prompts import Prompt, completion, run_prompt
from chalk.queries.named_query import NamedQuery
from chalk.queries.query_context import ChalkContext
from chalk.queries.scheduled_query import ScheduledQuery
from chalk.sql import make_sql_file_resolver
from chalk.state import State
from chalk.streams import MaterializationWindowConfig, Windowed, group_by_windowed, stream, windowed
from chalk.utils import AnyDataclass
from chalk.utils.duration import CronTab, Duration, ScheduleOptions
from chalk.utils.json import JSON

batch = offline
realtime = online
embedding = embed

__all__ = (
    "AnyDataclass",
    "BranchId",
    "ChalkContext",
    "Cron",
    "CronTab",
    "DataFrame",
    "Distance",
    "Document",
    "Duration",
    "EnvironmentId",
    "Environments",
    "FeatureTime",
    "Features",
    "JSON",
    "Last",
    "MaterializationWindowConfig",
    "NamedQuery",
    "Now",
    "OfflineResolver",
    "OnlineResolver",
    "Primary",
    "Prompt",
    "Resolver",
    "ScheduleOptions",
    "ScheduledQuery",
    "State",
    "StaticOperator",
    "Tags",
    "Validation",
    "Windowed",
    "_",
    "__",
    "__version__",
    "after",
    "batch",
    "before",
    "chalk_logger",
    "completion",
    "description",
    "embed",
    "embedding",
    "feature",
    "freeze_time",
    "get_resolver",
    "group_by_windowed",
    "has_many",
    "has_one",
    "is_primary",
    "make_sql_file_resolver",
    "offline",
    "online",
    "op",
    "owner",
    "realtime",
    "run_prompt",
    "scan_parquet",
    "stream",
    "tags",
    "underscore",
    "windowed",
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/engine/v1/offline_store_service.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.common.v1 import feature_values_pb2 as chalk_dot_common_dot_v1_dot_feature__values__pb2
from chalk._gen.chalk.common.v1 import query_log_pb2 as chalk_dot_common_dot_v1_dot_query__log__pb2
from chalk._gen.chalk.common.v1 import query_values_pb2 as chalk_dot_common_dot_v1_dot_query__values__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n+chalk/engine/v1/offline_store_service.proto\x12\x0f\x63halk.engine.v1\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a$chalk/common/v1/feature_values.proto\x1a\x1f\x63halk/common/v1/query_log.proto\x1a"chalk/common/v1/query_values.proto"\x1a\n\x18OfflineStoreServiceDummy2\xba\x05\n\x13OfflineStoreService\x12r\n\x12GetQueryLogEntries\x12*.chalk.common.v1.GetQueryLogEntriesRequest\x1a+.chalk.common.v1.GetQueryLogEntriesResponse"\x03\x80}\x04\x12\x66\n\x0eGetQueryValues\x12&.chalk.common.v1.GetQueryValuesRequest\x1a\'.chalk.common.v1.GetQueryValuesResponse"\x03\x80}\x04\x12~\n\x15GetFeatureValuesChart\x12-.chalk.common.v1.GetFeatureValuesChartRequest\x1a..chalk.common.v1.GetFeatureValuesChartResponse"\x06\x88\x02\x01\x80}\x04\x12\x9c\x01\n\x1fGetFeatureValuesTimeSeriesChart\x12\x37.chalk.common.v1.GetFeatureValuesTimeSeriesChartRequest\x1a\x38.chalk.common.v1.GetFeatureValuesTimeSeriesChartResponse"\x06\x88\x02\x01\x80}\x04\x12\xa2\x01\n!GetFeatureValuesTimeSeriesChartV2\x12\x39.chalk.common.v1.GetFeatureValuesTimeSeriesChartV2Request\x1a:.chalk.common.v1.GetFeatureValuesTimeSeriesChartV2Response"\x06\x88\x02\x01\x80}\x04\x1a\x03\x88\x02\x01\x42\x8d\x01\n\x13\x63om.chalk.engine.v1B\x18OfflineStoreServiceProtoP\x01\xa2\x02\x03\x43\x45X\xaa\x02\x0f\x43halk.Engine.V1\xca\x02\x0f\x43halk\\Engine\\V1\xe2\x02\x1b\x43halk\\Engine\\V1\\GPBMetadata\xea\x02\x11\x43halk::Engine::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.engine.v1.offline_store_service_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.engine.v1B\030OfflineStoreServiceProtoP\001\242\002\003CEX\252\002\017Chalk.Engine.V1\312\002\017Chalk\\Engine\\V1\342\002\033Chalk\\Engine\\V1\\GPBMetadata\352\002\021Chalk::Engine::V1"
    _globals["_OFFLINESTORESERVICE"]._options = None
    _globals["_OFFLINESTORESERVICE"]._serialized_options = b"\210\002\001"
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetQueryLogEntries"]._options = None
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetQueryLogEntries"]._serialized_options = b"\200}\004"
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetQueryValues"]._options = None
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetQueryValues"]._serialized_options = b"\200}\004"
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetFeatureValuesChart"]._options = None
    _globals["_OFFLINESTORESERVICE"].methods_by_name[
        "GetFeatureValuesChart"
    ]._serialized_options = b"\210\002\001\200}\004"
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetFeatureValuesTimeSeriesChart"]._options = None
    _globals["_OFFLINESTORESERVICE"].methods_by_name[
        "GetFeatureValuesTimeSeriesChart"
    ]._serialized_options = b"\210\002\001\200}\004"
    _globals["_OFFLINESTORESERVICE"].methods_by_name["GetFeatureValuesTimeSeriesChartV2"]._options = None
    _globals["_OFFLINESTORESERVICE"].methods_by_name[
        "GetFeatureValuesTimeSeriesChartV2"
    ]._serialized_options = b"\210\002\001\200}\004"
    _globals["_OFFLINESTORESERVICEDUMMY"]._serialized_start = 204
    _globals["_OFFLINESTORESERVICEDUMMY"]._serialized_end = 230
    _globals["_OFFLINESTORESERVICE"]._serialized_start = 233
    _globals["_OFFLINESTORESERVICE"]._serialized_end = 931
# @@protoc_insertion_point(module_scope)



from chalk._gen.chalk.common.v1 import query_log_pb2 as _query_log_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class OperationIdTableIdentifier(_message.Message):
    __slots__ = ("operation_id",)
    OPERATION_ID_FIELD_NUMBER: _ClassVar[int]
    operation_id: str
    def __init__(self, operation_id: _Optional[str] = ...) -> None: ...

class TableNameTableIdentifier(_message.Message):
    __slots__ = ("table_name", "filters")
    TABLE_NAME_FIELD_NUMBER: _ClassVar[int]
    FILTERS_FIELD_NUMBER: _ClassVar[int]
    table_name: str
    filters: _query_log_pb2.QueryLogFilters
    def __init__(
        self,
        table_name: _Optional[str] = ...,
        filters: _Optional[_Union[_query_log_pb2.QueryLogFilters, _Mapping]] = ...,
    ) -> None: ...

class GetQueryValuesPageToken(_message.Message):
    __slots__ = ("query_timestamp_hwm", "operation_id_hwm", "row_id_hwm")
    QUERY_TIMESTAMP_HWM_FIELD_NUMBER: _ClassVar[int]
    OPERATION_ID_HWM_FIELD_NUMBER: _ClassVar[int]
    ROW_ID_HWM_FIELD_NUMBER: _ClassVar[int]
    query_timestamp_hwm: _timestamp_pb2.Timestamp
    operation_id_hwm: str
    row_id_hwm: int
    def __init__(
        self,
        query_timestamp_hwm: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        operation_id_hwm: _Optional[str] = ...,
        row_id_hwm: _Optional[int] = ...,
    ) -> None: ...

class GetQueryValuesRequest(_message.Message):
    __slots__ = (
        "operation_id_identifier",
        "table_name_identifier",
        "query_timestamp_lower_bound_inclusive",
        "query_timestamp_upper_bound_exclusive",
        "features",
        "page_size",
        "page_token",
    )
    OPERATION_ID_IDENTIFIER_FIELD_NUMBER: _ClassVar[int]
    TABLE_NAME_IDENTIFIER_FIELD_NUMBER: _ClassVar[int]
    QUERY_TIMESTAMP_LOWER_BOUND_INCLUSIVE_FIELD_NUMBER: _ClassVar[int]
    QUERY_TIMESTAMP_UPPER_BOUND_EXCLUSIVE_FIELD_NUMBER: _ClassVar[int]
    FEATURES_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    operation_id_identifier: OperationIdTableIdentifier
    table_name_identifier: TableNameTableIdentifier
    query_timestamp_lower_bound_inclusive: _timestamp_pb2.Timestamp
    query_timestamp_upper_bound_exclusive: _timestamp_pb2.Timestamp
    features: _containers.RepeatedScalarFieldContainer[str]
    page_size: int
    page_token: str
    def __init__(
        self,
        operation_id_identifier: _Optional[_Union[OperationIdTableIdentifier, _Mapping]] = ...,
        table_name_identifier: _Optional[_Union[TableNameTableIdentifier, _Mapping]] = ...,
        query_timestamp_lower_bound_inclusive: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        query_timestamp_upper_bound_exclusive: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        features: _Optional[_Iterable[str]] = ...,
        page_size: _Optional[int] = ...,
        page_token: _Optional[str] = ...,
    ) -> None: ...

class GetQueryValuesResponse(_message.Message):
    __slots__ = ("next_page_token", "parquet")
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    PARQUET_FIELD_NUMBER: _ClassVar[int]
    next_page_token: str
    parquet: bytes
    def __init__(self, next_page_token: _Optional[str] = ..., parquet: _Optional[bytes] = ...) -> None: ...



from __future__ import annotations

import base64
import dataclasses
import os
import tempfile
from typing import TYPE_CHECKING

from chalk.utils.log_with_context import get_logger
from chalk.utils.missing_dependency import missing_dependency_exception

if TYPE_CHECKING:
    from google.auth.credentials import Credentials


_logger = get_logger(__name__)


@dataclasses.dataclass
class GCPProjectToken:
    credentials: Credentials
    project: str | None


def _base64_decode_str(s: str) -> str:
    return base64.b64decode(s).decode("utf-8")


def _read_b64_credentials(filename: str) -> str:
    with open(filename, "r") as creds:
        return _base64_decode_str(creds.read())


def _read_raw_json_credentials(filename: str) -> str:
    with open(filename, "r") as creds:
        return creds.read()


def get_credentials_from_file_base64(creds_filepath: str):
    try:
        import google.auth
    except ImportError:
        raise missing_dependency_exception("chalkpy[runtime]")
    with tempfile.TemporaryDirectory() as tmp_dir:
        temp_filename = os.path.join(tmp_dir, "credentials.json")
        with open(temp_filename, "w") as temp_creds_file:
            try:
                # first try to base64 decode the creds. But maybe in staging they are raw JSON...
                creds = _read_b64_credentials(creds_filepath)
            except Exception:
                # so if we get an exception here just try to read the raw file without decoding
                creds = _read_raw_json_credentials(creds_filepath)
            temp_creds_file.write(creds)

        creds, project = google.auth.load_credentials_from_file(temp_filename)
        return GCPProjectToken(project=project, credentials=creds)



from typing import Union

from chalk._monitoring.Chart import (
    AlertSeverityKind,
    Chart,
    MetricFormulaKind,
    SeriesBase,
    ThresholdPosition,
    Trigger,
    _DatasetFeatureOperand,
    _Formula,
    _MultiSeriesOperand,
    _SingleSeriesOperand,
)
from chalk._monitoring.charts_enums_codegen import (
    ComparatorKind,
    FilterKind,
    GroupByKind,
    MetricKind,
    WindowFunctionKind,
)
from chalk._monitoring.charts_series_base import MetricFilter
from chalk.parsed.duplicate_input_gql import (
    AlertSeverityKindGQL,
    ComparatorKindGQL,
    CreateAlertTriggerGQL,
    CreateChartGQL,
    CreateDatasetFeatureOperandGQL,
    CreateMetricConfigGQL,
    CreateMetricConfigSeriesGQL,
    CreateMetricFilterGQL,
    CreateMetricFormulaGQL,
    FilterKindGQL,
    GroupByKindGQL,
    MetricFormulaKindGQL,
    MetricKindGQL,
    ThresholdKindGQL,
    WindowFunctionKindGQL,
)


def _convert_series(series: SeriesBase, chart_name: str) -> CreateMetricConfigSeriesGQL:
    for validation_fn in series._validations:
        validation_fn()
    return CreateMetricConfigSeriesGQL(
        metric=convert_metric_kind(series._metric, chart_name, series._name),
        filters=[_convert_filter(series_filter) for series_filter in series._filters],
        name=series._name or series._default_name,
        windowFunction=_convert_window_function(series._window_function),
        groupBy=[_convert_group_by(group_by) for group_by in series._group_by],
    )


def convert_metric_kind(metric_kind: Union[MetricKind, None], chart_name: str, series_name: str) -> MetricKindGQL:
    if not metric_kind:
        raise ValueError(
            f"Chart '{chart_name}' has a series '{series_name}' with no metric. "
            f"'metric' is a required value for Series instances"
        )
    return MetricKindGQL(metric_kind.value.upper())


def _convert_filter(filter: MetricFilter) -> CreateMetricFilterGQL:
    return CreateMetricFilterGQL(
        kind=_convert_filter_kind(filter.kind),
        comparator=_convert_comparator_kind(filter.comparator),
        value=filter.value,
    )


def _convert_filter_kind(filter_kind: FilterKind) -> FilterKindGQL:
    return FilterKindGQL(filter_kind.value.upper())


def _convert_comparator_kind(comparator_kind: ComparatorKind) -> ComparatorKindGQL:
    return ComparatorKindGQL(comparator_kind.value.upper())


def _convert_window_function(window_function: Union[WindowFunctionKind, None]) -> Union[WindowFunctionKindGQL, None]:
    if not window_function:
        return None
    return WindowFunctionKindGQL(window_function.value.upper())


def _convert_group_by(group_by: GroupByKind) -> GroupByKindGQL:
    return GroupByKindGQL(group_by.value.upper())


def _convert_formula(formula: _Formula, chart_name: str) -> CreateMetricFormulaGQL:
    operands = formula._operands
    if not operands:
        raise ValueError(
            f"Chart '{chart_name}' has a formula '{formula._name}' with no operands. "
            f"'operands' is a required value for Formula instances"
        )
    single_series = None
    multi_series = None
    dataset_feature = None
    if isinstance(operands, _SingleSeriesOperand):
        single_series = operands.operand
    elif isinstance(operands, _MultiSeriesOperand):
        multi_series = operands.operands
    elif isinstance(operands, _DatasetFeatureOperand):
        dataset_feature = CreateDatasetFeatureOperandGQL(dataset=operands.dataset, feature=operands.feature)
    if not single_series and not multi_series and not dataset_feature:
        raise ValueError(
            f"Chart '{chart_name}' has a formula {formula._name}' with an invalid value for operand '{operands}'"
        )
    return CreateMetricFormulaGQL(
        kind=_convert_metric_formula_kind(formula._kind, chart_name, formula._name),
        singleSeriesOperands=single_series,
        multiSeriesOperands=multi_series,
        datasetFeatureOperands=dataset_feature,
        name=formula._name,
    )


def _convert_metric_formula_kind(
    kind: Union[MetricFormulaKind, None], chart_name: str, formula_name: str
) -> MetricFormulaKindGQL:
    if not kind:
        raise ValueError(
            f"Chart '{chart_name}' has a formula '{formula_name}' with no operation kind. "
            f"'kind' is a required value for Formula instances"
        )
    return MetricFormulaKindGQL(kind.value.upper())


def _convert_trigger(trigger: Trigger, chart_name: str) -> CreateAlertTriggerGQL:
    if not trigger._name:
        raise ValueError(
            f"Chart {chart_name} has a trigger with no name. " f"'name' is a required value for Trigger instances"
        )
    if trigger._threshold_value is None:
        raise ValueError(
            "Chart {chart_name} has a trigger with no threshold value. "
            "'threshold_value' is a required value for Trigger instances"
        )
    return CreateAlertTriggerGQL(
        name=trigger._name,
        severity=_convert_severity(trigger._severity, chart_name, trigger._name),
        thresholdPosition=_convert_threshold_position(trigger._threshold_position, chart_name, trigger._name),
        thresholdValue=trigger._threshold_value,
        seriesName=trigger._series_name,
        channelName=trigger._channel_name,
        description=trigger._description,
    )


def _convert_severity(
    severity: Union[AlertSeverityKind, None], chart_name: str, trigger_name: str
) -> AlertSeverityKindGQL:
    if not severity:
        raise ValueError(
            f"Chart '{chart_name}' has a trigger '{trigger_name} with no severity level. "
            f"'severity' is a required value for Trigger instances"
        )
    return AlertSeverityKindGQL(severity.value.lower())  # this GQL Enum object is the only in lowercase


def _convert_threshold_position(
    threshold_position: Union[ThresholdPosition, None], chart_name: str, trigger_name: str
) -> ThresholdKindGQL:
    if not threshold_position:
        raise ValueError(
            f"Chart '{chart_name}' has a trigger '{trigger_name} with no threshold position. "
            f"'threshold_position' is a required value for Trigger instances"
        )
    return ThresholdKindGQL(threshold_position.value.upper())


def convert_chart(chart: Chart) -> CreateChartGQL:
    if not chart._window_period:
        raise ValueError(
            f"Chart {chart._name} has no window period. " f"window_period' is a required value for Chart instances"
        )
    config = CreateMetricConfigGQL(
        name=chart._name,
        windowPeriod=chart._window_period,
        series=[_convert_series(series, chart._name) for series in chart._series],
        formulas=[_convert_formula(formula, chart._name) for formula in chart._formulas],
        trigger=_convert_trigger(chart._trigger, chart._name) if chart._trigger else None,
    )
    return CreateChartGQL(
        id=str(hash(chart)),
        config=config,
        entityKind=chart._entity_kind.value,
        entityId=chart._entity_id,
    )



from __future__ import annotations

import datetime as dt
from typing import Literal, Mapping, get_args

from chalk.features._encoding.http import HttpResponse
from chalk.features.underscore import Underscore, UnderscoreFunction

_HttpMethod = Literal["GET", "POST", "PUT", "DELETE", "PATCH", "OPTIONS", "HEAD", "TRACE", "CONNECT"]
_HTTP_METHOD_LIST = get_args(_HttpMethod)


class UnderscoreHttpRequest(UnderscoreFunction):
    def __init__(
        self,
        url: Underscore | str,
        method: _HttpMethod,
        headers: Underscore | Mapping[str, str] | None = None,
        body: Underscore | str | bytes | None = None,
        *,
        allow_redirects: bool = False,
        timeout: dt.timedelta | float | None = None,
    ):
        if isinstance(url, str) and not url.startswith(("http://", "https://")):
            raise ValueError(f"F.http_request(): URL must start with http:// or https://, got {url}")
        if not isinstance(url, (str, Underscore)):  # pyright: ignore[reportUnnecessaryIsInstance]
            raise TypeError(f"F.http_request(): URL must be a string or Expression, got {type(url)}")
        if (
            not isinstance(body, (str, bytes, Underscore)) and body is not None
        ):  # pyright: ignore[reportUnnecessaryIsInstance]
            raise TypeError(f"F.http_request(): body must be a string, bytes, Expression, or None: got {type(body)}")
        if method not in _HTTP_METHOD_LIST:
            raise ValueError(f"F.http_request(): Unknown HTTP method {method}, Supported: {_HTTP_METHOD_LIST}")
        if headers is not None and not isinstance(headers, (dict, Underscore)):
            raise TypeError(f"F.http_request(): headers must be a dict or Underscore, got {type(headers)}")
        if isinstance(timeout, dt.timedelta):
            timeout = timeout.total_seconds()
        super().__init__(
            "http_request",
            url,
            method,
            headers,
            body,
            allow_redirects,
            int(timeout * 1000) if timeout is not None else None,
        )


def http_request(
    url: Underscore | str,
    method: _HttpMethod,
    headers: Underscore | Mapping[str, str] | None = None,
    body: Underscore | str | bytes | None = None,
    *,
    allow_redirects: bool = True,
    timeout: dt.timedelta | float | None = None,
):
    """
    Make an HTTP request. The return type of this function is a `HttpResponse`, and features that are the result of
    this function should be annotated as such.

    HttpResponse is considered to be a struct, and you may use struct accessing syntax to access fields, HttpResponse
    consists of

    - status_code: int
    - headers: map<string, string>
    - body: bytes or string
    - final_url: string

    Parameters
    ----------
    url
        The URL to send the request to. Must start with http:// or https://.
    method
        The HTTP method to use. Must be one of GET, POST, PUT, DELETE, PATCH, OPTIONS, HEAD, TRACE, CONNECT.
    headers
        The headers to include in the request, if any. Should take the form of a map of strings to strings.
        The Content-Length header is automatically added to the headers, unless you explicitly set it in the request
        headers.
    body
        The body of the request, if any. Should be a string or bytes.
    allow_redirects
        Whether to follow redirects. Defaults to True.
    timeout
        The timeout for the request, in seconds. If None or 0, no timeout will be set.
        Timeout's precision is limited to milliseconds.
        Defaults to None.

    Examples
    --------
    >>> from chalk import _
    >>> import chalk.functions as F
    >>> import datetime as dt
    >>> from chalk.functions.http import HttpResponse
    >>> @features
    ... class User:
    ...    id: str
    ...    json_data: str
    ...    resp: HttpResponse[str] = F.http_request(
    ...        "https://example.com",
    ...        "POST",
    ...        headers={"Content-Type": "application/json", "X-Client-Id": "abc123", "X-Client-Secret": "xyz789"},
    ...        body=_.json_data,
    ...        allow_redirects=True,
    ...        timeout=dt.timedelta(seconds=5),
    ...    )
    ...    status_code: int = _.resp.status_code
    ...    resp_body: str = _.resp.body
    """

    return UnderscoreHttpRequest(
        url=url,
        method=method,
        headers=headers,
        body=body,
        allow_redirects=allow_redirects,
        timeout=timeout,
    )


def http_get(
    url: Underscore | str,
    headers: Underscore | Mapping[str, str] | None = None,
    body: Underscore | str | bytes | None = None,
    *,
    allow_redirects: bool = True,
    timeout: dt.timedelta | float | None = None,
):
    """
    HTTP GET request. See `http_request` for more details.
    """
    return http_request(
        url=url,
        method="GET",
        headers=headers,
        body=body,
        allow_redirects=allow_redirects,
        timeout=timeout,
    )


def http_post(
    url: Underscore | str,
    headers: Underscore | Mapping[str, str] | None = None,
    body: Underscore | str | bytes | None = None,
    *,
    allow_redirects: bool = True,
    timeout: dt.timedelta | float | None = None,
):
    """
    HTTP POST request. See `http_request` for more details.
    """
    return http_request(
        url=url,
        method="POST",
        headers=headers,
        body=body,
        allow_redirects=allow_redirects,
        timeout=timeout,
    )


def http_put(
    url: Underscore | str,
    headers: Underscore | Mapping[str, str] | None = None,
    body: Underscore | str | bytes | None = None,
    *,
    allow_redirects: bool = True,
    timeout: dt.timedelta | float | None = None,
):
    """
    HTTP PUT request. See `http_request` for more details.
    """
    return http_request(
        url=url,
        method="PUT",
        headers=headers,
        body=body,
        allow_redirects=allow_redirects,
        timeout=timeout,
    )


def http_delete(
    url: Underscore | str,
    headers: Underscore | Mapping[str, str] | None = None,
    body: Underscore | str | bytes | None = None,
    *,
    allow_redirects: bool = True,
    timeout: dt.timedelta | float | None = None,
):
    """
    HTTP PUT request. See `http_request` for more details.
    """
    return http_request(
        url=url,
        method="DELETE",
        headers=headers,
        body=body,
        allow_redirects=allow_redirects,
        timeout=timeout,
    )


__all__ = [
    "HttpResponse",
    "UnderscoreHttpRequest",
    "http_request",
    "http_get",
    "http_post",
    "http_put",
    "http_delete",
]
